@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 from __future__ import with_statement
-from basetest import BaseTest, StringIO
+from basetest import BaseTest, StringIO, BytesIO
 import sys, tempfile, os
 import unittest
 import logging, warnings
@@ -14,6 +14,7 @@ os.environ["http_proxy"] = "localhost:8000"
 from zeroinstall import helpers
 from zeroinstall.injector import model, gpg, download, trust, background, arch, selections, qdom, run
 from zeroinstall.injector.requirements import Requirements
+from zeroinstall.injector.scheduler import Site
 from zeroinstall.injector.driver import Driver
 from zeroinstall.zerostore import NotStored
 from zeroinstall.support import basedir, tasks, ro_rmtree
@@ -23,6 +24,8 @@ import my_dbus
 
 import server
 
+mydir = os.path.dirname(os.path.abspath(__file__))
+
 ran_gui = False
 def raise_gui(*args, **kwargs):
 	global ran_gui
@@ -139,6 +142,14 @@ def run_server(*args):
 
 real_get_selections_gui = helpers.get_selections_gui
 
+# Count how many downloads we request so we can check it
+traced_downloads = None
+orig_download = Site.download
+def wrap_download(self, step, timeout = None):
+	traced_downloads.append(step.url)
+	return orig_download(self, step)
+Site.download = wrap_download
+
 class TestDownload(BaseTest):
 	def setUp(self):
 		BaseTest.setUp(self)
@@ -163,6 +174,9 @@ class TestDownload(BaseTest):
 		global ran_gui
 		ran_gui = False
 
+		global traced_downloads
+		traced_downloads = []
+
 	def tearDown(self):
 		# Wait for all downloads to finish, otherwise they may interfere with other tests
 		for dl in list(self.config.handler.monitored_downloads):
@@ -397,7 +411,7 @@ class TestDownload(BaseTest):
 		finally:
 			sys.stdout = old_out
 	
-	def testRename(self):
+	def testRecipeRename(self):
 		with output_suppressed():
 			run_server(('HelloWorld.tar.bz2',))
 			requirements = Requirements(os.path.abspath('RecipeRename.xml'))
@@ -410,6 +424,122 @@ class TestDownload(BaseTest):
 			assert not os.path.exists(os.path.join(path, 'HelloWorld'))
 			assert not os.path.exists(os.path.join(path, 'HelloUniverse', 'main'))
 
+	def testRecipeRenameToNewDest(self):
+		with output_suppressed():
+			run_server(('HelloWorld.tar.bz2',))
+			requirements = Requirements(os.path.abspath('RecipeRenameToNewDest.xml'))
+			requirements.command = None
+			driver = Driver(requirements = requirements, config = self.config)
+			driver_download(driver)
+			digests = driver.solver.selections[requirements.interface_uri].digests
+			path = self.config.stores.lookup_any(digests)
+			assert os.path.exists(os.path.join(path, 'HelloWorld', 'bin', 'main'))
+			assert not os.path.exists(os.path.join(path, 'HelloWorld', 'main'))
+
+	def testRecipeRemoveFile(self):
+		with output_suppressed():
+			run_server(('HelloWorld.tar.bz2',))
+			requirements = Requirements(os.path.abspath('RecipeRemove.xml'))
+			requirements.command = None
+			driver = Driver(requirements = requirements, config = self.config)
+			driver_download(driver)
+			digests = driver.solver.selections[requirements.interface_uri].digests
+			path = self.config.stores.lookup_any(digests)
+			assert os.path.exists(os.path.join(path, 'HelloWorld'))
+			assert not os.path.exists(os.path.join(path, 'HelloWorld', 'main'))
+
+	def testRecipeRemoveDir(self):
+		with output_suppressed():
+			run_server(('HelloWorld.tar.bz2',))
+			requirements = Requirements(os.path.abspath('RecipeRemoveDir.xml'))
+			requirements.command = None
+			driver = Driver(requirements = requirements, config = self.config)
+			driver_download(driver)
+			digests = driver.solver.selections[requirements.interface_uri].digests
+			path = self.config.stores.lookup_any(digests)
+			assert not os.path.exists(os.path.join(path, 'HelloWorld'))
+
+	def testRecipeExtractToNewSubdirectory(self):
+		with output_suppressed():
+			run_server(('HelloWorld.tar.bz2',))
+			requirements = Requirements(os.path.abspath('RecipeExtractToNewDest.xml'))
+			requirements.command = None
+			driver = Driver(requirements = requirements, config = self.config)
+			driver_download(driver)
+			digests = driver.solver.selections[requirements.interface_uri].digests
+			path = self.config.stores.lookup_any(digests)
+			assert os.path.exists(os.path.join(path, 'src', 'HelloWorld', 'main'))
+
+	def testRecipeExtractToExistingSubdirectory(self):
+		with output_suppressed():
+			run_server(('HelloWorld.tar.bz2','HelloWorld.tar.bz2'))
+			requirements = Requirements(os.path.abspath('RecipeExtractToExistingDest.xml'))
+			requirements.command = None
+			driver = Driver(requirements = requirements, config = self.config)
+			driver_download(driver)
+			digests = driver.solver.selections[requirements.interface_uri].digests
+			path = self.config.stores.lookup_any(digests)
+			assert os.path.exists(os.path.join(path, 'HelloWorld', 'main')) # first archive's main
+			assert os.path.exists(os.path.join(path, 'HelloWorld', 'HelloWorld', 'main')) # second archive, extracted to HelloWorld/
+
+	def testRecipeLocal(self):
+		recipe = model.Recipe()
+		blocker = self.config.fetcher.cook("sha256new_4OYMIQUY7QOBJGX36TEJS35ZEQT24QPEMSNZGTFESWMRW6CSXBKQ", recipe, self.config.stores)
+		tasks.wait_for_blocker(blocker)
+
+		try:
+			recipe.steps.append(model.RemoveStep("."))
+			blocker = self.config.fetcher.cook("sha256new_XXX", recipe, self.config.stores)
+			tasks.wait_for_blocker(blocker)
+			assert 0
+		except model.SafeException as ex:
+			assert "path '.' is not within the base directory" in str(ex), ex
+
+	def testRenameFailure(self):
+		recipe = model.Recipe()
+		try:
+			recipe.steps.append(model.RenameStep("missing-source", "dest"))
+			blocker = self.config.fetcher.cook("sha256new_XXX", recipe, self.config.stores)
+			tasks.wait_for_blocker(blocker)
+			assert 0
+		except model.SafeException as ex:
+			assert "<rename> source 'missing-source' does not exist" in str(ex), ex
+
+	def testRecipeSingleFile(self):
+		with output_suppressed():
+			run_server(('HelloWorldMain',))
+			requirements = Requirements(os.path.abspath('RecipeSingleFile.xml'))
+			requirements.command = None
+			driver = Driver(requirements = requirements, config = self.config)
+			driver_download(driver)
+			digests = driver.solver.selections[requirements.interface_uri].digests
+			path = self.config.stores.lookup_any(digests)
+			with open(os.path.join(path, 'bin','main'), 'rt') as stream:
+				assert 'Hello World' in stream.read()
+
+	def testExtractToNewSubdirectory(self):
+		with output_suppressed():
+			run_server(('HelloWorld.tar.bz2',))
+			requirements = Requirements(os.path.abspath('HelloExtractToNewDest.xml'))
+			requirements.command = None
+			driver = Driver(requirements = requirements, config = self.config)
+			driver_download(driver)
+			digests = driver.solver.selections[requirements.interface_uri].digests
+			path = self.config.stores.lookup_any(digests)
+			assert os.path.exists(os.path.join(path, 'src', 'HelloWorld', 'main'))
+
+	def testDownloadFile(self):
+		with output_suppressed():
+			run_server(('HelloWorldMain',))
+			requirements = Requirements(os.path.abspath('HelloSingleFile.xml'))
+			requirements.command = None
+			driver = Driver(requirements = requirements, config = self.config)
+			driver_download(driver)
+			digests = driver.solver.selections[requirements.interface_uri].digests
+			path = self.config.stores.lookup_any(digests)
+			with open(os.path.join(path, 'main'), 'rt') as stream:
+				assert 'Hello World' in stream.read()
+
 	def testSymlink(self):
 		old_out = sys.stdout
 		try:
@@ -501,6 +631,40 @@ class TestDownload(BaseTest):
 			path = self.config.stores.lookup_any(driver.solver.selections.selections['http://example.com:8000/Hello.xml'].digests)
 			assert os.path.exists(os.path.join(path, 'HelloWorld', 'main'))
 
+	def testImplMirrorFails(self):
+		with resourcewarnings_suppressed():
+			trust.trust_db.trust_key('DE937DD411906ACF7C263B396FCF121BE2390E0B', 'example.com:8000')
+			run_server('/Hello.xml',
+					'/6FCF121BE2390E0B.gpg',
+					server.Give404('/HelloWorld.tgz'),
+					server.Give404('/0mirror/archive/http%3A%23%23example.com%3A8000%23HelloWorld.tgz'),
+					server.Give404('/0mirror/feeds/http/example.com:8000/Hello.xml/impl/sha1=3ce644dc725f1d21cfcf02562c76f375944b266a'))
+			driver = Driver(requirements = Requirements('http://example.com:8000/Hello.xml'), config = self.config)
+			self.config.mirror = 'http://example.com:8000/0mirror'
+
+			refreshed = driver.solve_with_downloads()
+			tasks.wait_for_blocker(refreshed)
+			assert driver.solver.ready
+
+			getLogger().setLevel(logging.ERROR)
+			try:
+				downloaded = driver.download_uncached_implementations()
+				tasks.wait_for_blocker(downloaded)
+				assert 0
+			except download.DownloadError as ex:
+				assert 'Missing: HelloWorld.tgz' in str(ex), ex
+
+			self.assertEqual([
+				'http://example.com:8000/Hello.xml',
+				'http://example.com:8000/6FCF121BE2390E0B.gpg',
+				# The original archive:
+				'http://example.com:8000/HelloWorld.tgz',
+				# Mirror of original archive:
+				'http://example.com:8000/0mirror/archive/http%3A%23%23example.com%3A8000%23HelloWorld.tgz',
+				# Mirror of implementation:
+				'http://example.com:8000/0mirror/feeds/http/example.com:8000/Hello.xml/impl/sha1=3ce644dc725f1d21cfcf02562c76f375944b266a'
+				], traced_downloads)
+
 	def testLocalFeedMirror(self):
 		with resourcewarnings_suppressed():
 			# This is like testImplMirror, except we have a local feed.
@@ -798,6 +962,18 @@ class TestDownload(BaseTest):
 		with trapped_exit(1):
 			sels = app.get_selections(may_update = True)
 		kill_server_process()
+	
+	def testChunked(self):
+		if sys.version_info[0] < 3:
+			return	# not a problem with Python 2
+		run_server('chunked')
+		dl = self.config.fetcher.download_url('http://localhost/chunked')
+		tmp = dl.tempfile
+		tasks.wait_for_blocker(dl.downloaded)
+		tasks.check(dl.downloaded)
+		tmp.seek(0)
+		self.assertEqual(b'hello world', tmp.read())
+		kill_server_process()
 
 	def testAbort(self):
 		dl = download.Download("http://localhost/test.tgz", auto_delete = True)
@@ -813,7 +989,6 @@ class TestDownload(BaseTest):
 		assert dl.tempfile is None
 
 	def testDownloadIconFails(self):
-		mydir = os.path.dirname(os.path.abspath(__file__))
 		path = model.canonical_iface_uri(os.path.join(mydir, 'Binary.xml'))
 		iface = self.config.iface_cache.get_interface(path)
 		blocker = self.config.fetcher.download_icon(iface)
@@ -822,6 +997,72 @@ class TestDownload(BaseTest):
 			assert False
 		except download.DownloadError as ex:
 			assert "Error downloading http://localhost/missing.png" in str(ex), ex
+	
+	def testSearch(self):
+		out, err = self.run_0install(['search'])
+		assert out.lower().startswith("usage:")
+		assert 'QUERY' in out, out
+
+		run_server('/0mirror/search/')
+		self.config.mirror = 'http://example.com:8000/0mirror'
+		out, err = self.run_0install(['search', 'firefox'])
+		kill_server_process()
+		self.assertEqual("", err)
+		assert 'Firefox - Webbrowser' in out, out
+
+	def testLocalArchive(self):
+		local_iface = os.path.join(mydir, 'LocalArchive.xml')
+		with open(local_iface, 'rb') as stream:
+			root = qdom.parse(stream)
+
+		# Not local => error
+		feed = model.ZeroInstallFeed(root)
+		impl = feed.implementations['impl1']
+		blocker = self.config.fetcher.download_impls([impl], self.config.stores)
+		try:
+			tasks.wait_for_blocker(blocker)
+			assert 0
+		except model.SafeException as ex:
+			assert "Relative URL 'HelloWorld.tgz' in non-local feed" in str(ex), ex
+
+		feed = model.ZeroInstallFeed(root, local_path = local_iface)
+
+		# Missing file
+		impl2 = feed.implementations['impl2']
+		blocker = self.config.fetcher.download_impls([impl2], self.config.stores)
+		try:
+			tasks.wait_for_blocker(blocker)
+			assert 0
+		except model.SafeException as ex:
+			assert 'tests/IDONTEXIST.tgz' in str(ex), ex
+
+		# Wrong size
+		impl3 = feed.implementations['impl3']
+		blocker = self.config.fetcher.download_impls([impl3], self.config.stores)
+		try:
+			tasks.wait_for_blocker(blocker)
+			assert 0
+		except model.SafeException as ex:
+			assert 'feed says 177, but actually 176 bytes' in str(ex), ex
+
+		self.config.network_use = model.network_offline
+		r = Requirements(local_iface)
+		r.command = None
+		driver = Driver(requirements = r, config = self.config)
+		driver.need_download()
+		assert driver.solver.ready, driver.solver.get_failure_reason()
+
+		# Local => OK
+		impl = feed.implementations['impl1']
+
+		path = self.config.stores.lookup_maybe(impl.digests)
+		assert not path
+
+		blocker = self.config.fetcher.download_impls([impl], self.config.stores)
+		tasks.wait_for_blocker(blocker)
+
+		path = self.config.stores.lookup_any(impl.digests)
+		assert os.path.exists(os.path.join(path, 'HelloWorld'))
 
 if __name__ == '__main__':
 	try:
