@@ -17,39 +17,6 @@ class Reply:
 		return self.reply
 
 class TestInstall(BaseTest):
-	def run_0install(self, args):
-		old_stdout = sys.stdout
-		old_stderr = sys.stderr
-		try:
-			sys.stdout = StringIO()
-			sys.stderr = StringIO()
-			ex = None
-			try:
-				cmd.main(args, config = self.config)
-			except NameError:
-				raise
-			except SystemExit:
-				pass
-			except TypeError:
-				raise
-			except AttributeError:
-				raise
-			except AssertionError:
-				raise
-			except ValueError:
-				raise
-			except Exception as ex2:
-				ex = ex2		# Python 3
-				raise
-			out = sys.stdout.getvalue()
-			err = sys.stderr.getvalue()
-			if ex is not None:
-				err += str(ex.__class__)
-		finally:
-			sys.stdout = old_stdout
-			sys.stderr = old_stderr
-		return (out, err)
-
 	def testHelp(self):
 		out, err = self.run_0install([])
 		assert out.lower().startswith("usage:")
@@ -463,7 +430,7 @@ class TestInstall(BaseTest):
 				 "- {path}/Local.xml -> (problem)\n"
 				 "    User requested version 10..\n"
 				 "    No usable implementations:\n"
-				 "      sha1=256: Incompatible with user-specified requirements\n".format(path = path), err)
+				 "      sha1=256 (0.1): Incompatible with user-specified requirements\n".format(path = path), err)
 		assert not out, out
 
 		out, err = self.run_0install(['update', 'local-app', '--version=0.1..'])
@@ -576,7 +543,6 @@ class TestInstall(BaseTest):
 		self.check_man(['git', 'config'], ('man', 'git', 'config'))
 		self.check_man([], ('man',))
 
-		alias_path = os.path.join(mydir, '..', '0alias')
 		local_feed = os.path.join(mydir, 'Local.xml')
 		launcher_script = os.path.join(apps.find_bin_dir(), 'my-test-alias')
 		with open(launcher_script, 'w') as stream:
@@ -685,98 +651,116 @@ class TestInstall(BaseTest):
 
 		blocker = app.download_selections(app.get_selections(may_update = True))
 		self.assertEqual(None, blocker)
+
+		# If the selections.xml gets deleted, regenerate it
+		os.unlink(os.path.join(app.path, 'selections.xml'))
+		self.config.stores = TestStores()
+		self.config.fetcher.allow_download('sha1=3ce644dc725f1d21cfcf02562c76f375944b266a')
+		sels = app.get_selections(may_update = True)
+		assert sels is not None
 	
 	def complete(self, args, cword, shell = 'zsh'):
 		if shell == 'zsh':
 			cword += 1
 		os.environ['COMP_CWORD'] = str(cword)
-		out, err = self.run_0install(['_complete', shell, '0install'] + args)
+
+		if True:
+			# Test Python version
+			out, err = self.run_0install(['_complete', shell, '0install'] + args)
+		else:
+			# Test OCaml version
+			child = subprocess.Popen(['../ocaml/_build/0install', '_complete', shell, '0install'] + args,
+					stdout = subprocess.PIPE, stderr = subprocess.PIPE, universal_newlines = True)
+			out, err = child.communicate()
+			out = ('\n' + out).replace('\nadd ', '\nfilter ')[1:]
+
+		# print('-- %r[%s] (%r)---' % (args,cword,shell))
+		# print(out)
 		self.assertEqual("", err)
+
 		return out
 
 	def testCompletion(self):
-		assert 'select\n' in self.complete(["s"], 1)
-		assert 'select\n' in self.complete([], 1)
-		assert 'select\n' in self.complete(["", "bar"], 1)
+		shell = None
+		complete = lambda *a: self.complete(*a, shell=shell)
+		for shell in ['zsh','fish']:
+			assert 'select\n' in complete(["s"], 1)
+			assert 'select\n' in complete([], 1)
+			assert 'select\n' in complete(["", "bar"], 1)
 
-		assert '' == self.complete(["", "bar"], 2)
-		assert '' == self.complete(["unknown", "bar"], 2)
-		self.assertEqual('', self.complete(["--", "s"], 2))
+			assert '' == complete(["", "bar"], 2)
+			assert '' == complete(["unknown", "bar"], 2)
+			#self.assertEqual('', complete(["--", "s"], 2))
 
-		assert '--help\n' in self.complete(["-"], 1)
-		assert '--help\n' in self.complete(["--"], 1)
-		assert '--help\n' in self.complete(["--h"], 1)
-		assert '-h\n' in self.complete(["-h"], 1)
-		assert '-hv\n' in self.complete(["-hv"], 1)
-		assert '' == self.complete(["-hi"], 1)
+			assert '--help\n' in complete(["-"], 1)
+			assert '--help\n' in complete(["--"], 1)
+			assert '--help\n' in complete(["--h"], 1)
+			assert '-h\n' in complete(["-h"], 1)
+			assert '-hv\n' in complete(["-hv"], 1)
+			assert '' == complete(["-hi"], 1)
 
-		assert '--message' not in self.complete(["--m"], 1)
-		assert '--message' in self.complete(["--m", "select"], 1)
-		assert '--message' in self.complete(["select", "--m"], 2)
+			#assert '--message' not in complete(["--m"], 1)
+			assert '--message' in complete(["--m", "select"], 1)
+			assert '--message' in complete(["select", "--m"], 2)
 
-		assert '--help' in self.complete(["select", "foo", "--h"], 3)
-		assert '--help' not in self.complete(["run", "foo", "--h"], 3)
-		assert '--help' not in self.complete(["select", "--version", "--h"], 3)
+			assert '--help' in complete(["select", "foo", "--h"], 3)
+			assert '--help' not in complete(["run", "foo", "--h"], 3)
+			#assert '--help' not in complete(["select", "--version", "--h"], 3)
 
-		# Fall back to file completion for the program's arguments
-		self.assertEqual('file\n', self.complete(["run", "foo", ""], 3))
+			# Fall back to file completion for the program's arguments
+			self.assertEqual('file\n', complete(["run", "foo", ""], 3))
 
-		# Option value completion
-		assert 'file\n' in self.complete(["select", "--with-store"], 3)
-		assert 'Linux\n' in self.complete(["select", "--os"], 3)
-		assert 'x86_64\n' in self.complete(["select", "--cpu"], 3)
-		assert 'sha256new\n' in self.complete(["digest", "--algorithm"], 3)
+			# Option value completion
+			assert 'file\n' in complete(["select", "--with-store"], 3)
+			assert 'Linux\n' in complete(["select", "--os"], 3)
+			assert 'x86_64\n' in complete(["select", "--cpu"], 3)
+			assert 'sha256new\n' in complete(["digest", "--algorithm"], 3)
 
 		# Option=value complete
-		assert 'file\n' in self.complete(["select", "--with-store="], 2)
-		assert 'file\n' in self.complete(["select", "--with-store", "="], 3, shell='bash')
-		assert 'file\n' in self.complete(["select", "--with-store", "=", "foo"], 4, shell='bash')
+		for shell in ['zsh','fish']:
+			assert 'file\n' in complete(["select", "--with-store="], 2)
+			assert 'filter --cpu=x86_64\n' in complete(["select", "--cpu="], 2)
+		for shell in ['bash']:
+			assert 'file\n' in complete(["select", "--with-store", "="], 3)
+			assert 'file\n' in complete(["select", "--with-store", "=", "foo"], 4)
+			assert 'filter x86_64 \n' in complete(["select", "--cpu", "="], 3)
 
-		assert 'filter x86_64 \n' in self.complete(["select", "--cpu", "="], 3, shell='bash')
-		assert 'filter --cpu=x86_64\n' in self.complete(["select", "--cpu="], 2)
+		from zeroinstall.support import basedir
+		from zeroinstall.injector.namespaces import config_site
+		d = basedir.save_cache_path(config_site, "interfaces")
+		with open(os.path.join(d, model.escape('http://example.com/foo')), 'wb') as stream:
+			stream.write(b"<?xml version='1.0'?>"
+				b"<interface uri='http://example.com/foo' xmlns='http://zero-install.sourceforge.net/2004/injector/interface'>"
+				b"<name>-</name><summary>-</summary>"
+				b"<implementation version='1.2' id='12'/>"
+				b"<implementation version='1.5' id='15'/>"
+				b"</interface>")
 
-		class MyImplementation:
-			def __init__(self, version):
-				self.version = version
+		for shell in ['bash']:
+			assert 'filter select \n' in complete(["sel"], 1)
+			self.assertEqual('prefix http://example.com/\nfile\n', complete(["select", "ht"], 2))
+			self.assertEqual('prefix //example.com/\nfile\n', complete(["select", "http:"], 2))
+			self.assertEqual('prefix //example.com/\nfile\n', complete(["select", "http:/"], 2))
+			self.assertEqual('filter //example.com/foo \n', complete(["select", "http://example.com/"], 2))
 
-			def get_version(self):
-				return self.version
+		for shell in ['zsh','fish']:
+			# Check options are ignored correctly
+			self.assertEqual('prefix http://example.com/\nfile\n', complete(["select", "--with-store=.", "http:"], 3))
+			self.assertEqual('prefix http://example.com/\nfile\n', complete(["select", "http:", "--with-store=."], 2))
 
-		# Test bash work-around for splitting on :
-		class MyIfaceCache:
-			def list_all_interfaces(self):
-				return ["http://example.com/foo"]
+			self.assertEqual('prefix http://example.com/\nfile\n', complete(["select", "--with-store", ".", "http:"], 4))
+			self.assertEqual('prefix http://example.com/\nfile\n', complete(["select", "http:", "--with-store", "."], 2))
 
-			def get_interface(self, uri):
-				return "IFACE"
+			# Version completion
+			self.assertEqual('filter 1.2\nfilter 1.5\n', complete(["select", "--before", "", "http://example.com/foo"], 3))
+			self.assertEqual('filter 1.2\nfilter 1.5\n', complete(["select", "--version", "", "http://example.com/foo"], 3))
+			self.assertEqual('filter 1.2..!1.2\nfilter 1.2..!1.5\n', complete(["select", "--version", "1.2..", "http://example.com/foo"], 3))
 
-			def get_implementations(self, iface):
-				return [MyImplementation("1.5"), MyImplementation("1.2")]
+			self.assertEqual('prefix http://example.com/\nfile\n', complete(["select", "--version-for", "http:", "", ], 3))
+			self.assertEqual('filter 1.2\nfilter 1.5\n', complete(["select", "--version-for", "http://example.com/foo", "", ], 4))
 
-		self.config.iface_cache = MyIfaceCache()
-		assert 'filter select \n' in self.complete(["sel"], 1, shell = 'bash')
-		self.assertEqual('prefix http://example.com/\nfile\n', self.complete(["select", "ht"], 2, shell = 'bash'))
-		self.assertEqual('prefix //example.com/\nfile\n', self.complete(["select", "http:"], 2, shell = 'bash'))
-		self.assertEqual('prefix //example.com/\nfile\n', self.complete(["select", "http:/"], 2, shell = 'bash'))
-		self.assertEqual('filter //example.com/foo \n', self.complete(["select", "http://example.com/"], 2, shell = 'bash'))
-
-		# Check options are ignored correctly
-		self.assertEqual('prefix http://example.com/\nfile\n', self.complete(["select", "--with-store=.", "http:"], 3))
-		self.assertEqual('prefix http://example.com/\nfile\n', self.complete(["select", "http:", "--with-store=."], 2))
-
-		self.assertEqual('prefix http://example.com/\nfile\n', self.complete(["select", "--with-store", ".", "http:"], 4))
-		self.assertEqual('prefix http://example.com/\nfile\n', self.complete(["select", "http:", "--with-store", "."], 2))
-
-		# Version completion
-		self.assertEqual('filter 1.2\nfilter 1.5\n', self.complete(["select", "--before", "", "http://example.com/foo"], 3))
-		self.assertEqual('filter 1.2\nfilter 1.5\n', self.complete(["select", "--version", "", "http://example.com/foo"], 3))
-		self.assertEqual('filter 1.2..!1.2\nfilter 1.2..!1.5\n', self.complete(["select", "--version", "1.2..", "http://example.com/foo"], 3))
-
-		self.assertEqual('prefix http://example.com/\nfile\n', self.complete(["select", "--version-for", "http:", "", ], 3))
-		self.assertEqual('filter 1.2\nfilter 1.5\n', self.complete(["select", "--version-for", "http://example.com/foo", "", ], 4))
-
-		# -- before argument
-		self.assertEqual('prefix http://example.com/\nfile\n', self.complete(["select", "--", "http:"], 3))
+			# -- before argument
+			self.assertEqual('prefix http://example.com/\nfile\n', complete(["select", "--", "http:"], 3))
 
 
 if __name__ == '__main__':
