@@ -9,13 +9,14 @@ Support for managing apps (as created with "0install add").
 from zeroinstall import _, SafeException, logger
 from zeroinstall.support import basedir, portable_rename
 from zeroinstall.injector import namespaces, selections, qdom, model
-import re, os, time, tempfile
+import re, os, time, tempfile, errno
 
 # Avoid characters that are likely to cause problems (reject : and ; everywhere
 # so that apps can be portable between POSIX and Windows).
 valid_name = re.compile(r'''^[^./\\:=;'"][^/\\:=;'"]*$''')
 
 def validate_name(name):
+	"""@type name: str"""
 	if name == '0install':
 		raise SafeException("Creating an app called '0install' would cause trouble; try e.g. '00install' instead")
 	if valid_name.match(name): return
@@ -30,7 +31,9 @@ def _export(name, value):
 
 def find_bin_dir(paths = None):
 	"""Find the first writable path in the list (default $PATH),
-	skipping /bin, /sbin and everything under /usr except /usr/local/bin"""
+	skipping /bin, /sbin and everything under /usr except /usr/local/bin
+	@type paths: [str] | None
+	@rtype: str"""
 	if paths is None:
 		paths = os.environ['PATH'].split(os.pathsep)
 	for path in paths:
@@ -57,7 +60,7 @@ _command_template = """#!/bin/sh
 exec 0install run {app} "$@"
 """
 
-class AppScriptInfo:
+class AppScriptInfo(object):
 	"""@since: 1.12"""
 	name = None
 	command = None
@@ -88,15 +91,18 @@ def parse_script_header(stream):
 	info.name = line.split()[3]
 	return info
 
-class App:
+class App(object):
 	def __init__(self, config, path):
+		"""@type path: str"""
 		self.config = config
 		self.path = path
 
 	def set_selections(self, sels, set_last_checked = True):
 		"""Store a new set of selections. We include today's date in the filename
 		so that we keep a history of previous selections (max one per day), in case
-		we want to to roll back later."""
+		we want to to roll back later.
+		@type sels: L{zeroinstall.injector.selections.Selections}
+		@type set_last_checked: bool"""
 		date = time.strftime('%Y-%m-%d')
 		sels_file = os.path.join(self.path, 'selections-{date}.xml'.format(date = date))
 		dom = sels.toDOM()
@@ -120,7 +126,11 @@ class App:
 		else:
 			if os.path.exists(sels_latest):
 				os.unlink(sels_latest)
-			os.symlink(os.path.basename(sels_file), sels_latest)
+			if os.name == "nt":
+				import shutil
+				shutil.copyfile(sels_file, sels_latest)
+			else:
+				os.symlink(os.path.basename(sels_file), sels_latest)
 
 		if set_last_checked:
 			self.set_last_checked()
@@ -128,10 +138,10 @@ class App:
 	def get_selections(self, snapshot_date = None, may_update = False, use_gui = None):
 		"""Load the selections.
 		If may_update is True then the returned selections will be cached and available.
-		@param may_update: whether to check for updates
-		@type may_update: bool
 		@param snapshot_date: get a historical snapshot
 		@type snapshot_date: (as returned by L{get_history}) | None
+		@param may_update: whether to check for updates
+		@type may_update: bool
 		@param use_gui: whether to use the GUI for foreground updates
 		@type use_gui: bool | None (never/always/if possible)
 		@return: the selections
@@ -141,8 +151,16 @@ class App:
 			sels_file = os.path.join(self.path, 'selections-' + snapshot_date + '.xml')
 		else:
 			sels_file = os.path.join(self.path, 'selections.xml')
-		with open(sels_file, 'rb') as stream:
-			sels = selections.Selections(qdom.parse(stream))
+
+		try:
+			with open(sels_file, 'rb') as stream:
+				sels = selections.Selections(qdom.parse(stream))
+		except IOError as ex:
+			if may_update and ex.errno == errno.ENOENT:
+				logger.info("App selections missing: %s", ex)
+				sels = None
+			else:
+				raise
 
 		if may_update:
 			sels = self._check_for_updates(sels, use_gui)
@@ -163,6 +181,7 @@ class App:
 
 	def download_selections(self, sels):
 		"""Download any missing implementations.
+		@type sels: L{zeroinstall.injector.selections.Selections}
 		@return: a blocker which resolves when all needed implementations are available
 		@rtype: L{tasks.Blocker} | None"""
 		return sels.download_missing(self.config)	# TODO: package impls
@@ -176,83 +195,96 @@ class App:
 		we update the app's selections and return the new selections.
 		If we can't use the current selections, we update in the foreground.
 		We also schedule a background update from time-to-time anyway.
+		@type sels: L{zeroinstall.injector.selections.Selections}
+		@type use_gui: bool
 		@return: the selections to use
 		@rtype: L{selections.Selections}"""
 		need_solve = False		# Rerun solver (cached feeds have changed)
 		need_update = False		# Update over the network
 
-		utime = self._get_mtime('last-checked', warn_if_missing = True)
-		last_solve = max(self._get_mtime('last-solve', warn_if_missing = False), utime)
+		if sels:
+			utime = self._get_mtime('last-checked', warn_if_missing = True)
+			last_solve = max(self._get_mtime('last-solve', warn_if_missing = False), utime)
 
-		# Ideally, this would return all the files which were inputs into the solver's
-		# decision. Currently, we approximate with:
-		# - the previously selected feed files (local or cached)
-		# - configuration files for the selected interfaces
-		# - the global configuration
-		# We currently ignore feeds and interfaces which were
-		# considered but not selected.
-		# Can yield None (ignored), paths or (path, mtime) tuples.
-		# If this throws an exception, we will log it and resolve anyway.
-		def get_inputs():
-			for sel in sels.selections.values():
-				logger.info("Checking %s", sel.feed)
-				feed = iface_cache.get_feed(sel.feed)
-				if not feed:
-					raise IOError("Input %s missing; update" % sel.feed)
-				else:
-					if feed.local_path:
-						yield feed.local_path
+			# Ideally, this would return all the files which were inputs into the solver's
+			# decision. Currently, we approximate with:
+			# - the previously selected feed files (local or cached)
+			# - configuration files for the selected interfaces
+			# - the global configuration
+			# We currently ignore feeds and interfaces which were
+			# considered but not selected.
+			# Can yield None (ignored), paths or (path, mtime) tuples.
+			# If this throws an exception, we will log it and resolve anyway.
+			def get_inputs():
+				for sel in sels.selections.values():
+					logger.info("Checking %s", sel.feed)
+
+					if sel.feed.startswith('distribution:'):
+						# If the package has changed version, we'll detect that below
+						# with get_unavailable_selections.
+						pass
+					elif os.path.isabs(sel.feed):
+						# Local feed
+						yield sel.feed
 					else:
-						yield (feed.url, feed.last_modified)
+						# Cached feed
+						cached = basedir.load_first_cache(namespaces.config_site, 'interfaces', model.escape(sel.feed))
+						if cached:
+							yield cached
+						else:
+							raise IOError("Input %s missing; update" % sel.feed)
 
-				# Per-feed configuration
-				yield basedir.load_first_config(namespaces.config_site, namespaces.config_prog,
-								   'interfaces', model._pretty_escape(sel.interface))
+					# Per-feed configuration
+					yield basedir.load_first_config(namespaces.config_site, namespaces.config_prog,
+									   'interfaces', model._pretty_escape(sel.interface))
 
-			# Global configuration
-			yield basedir.load_first_config(namespaces.config_site, namespaces.config_prog, 'global')
+				# Global configuration
+				yield basedir.load_first_config(namespaces.config_site, namespaces.config_prog, 'global')
 
-		# If any of the feeds we used have been updated since the last check, do a quick re-solve
-		iface_cache = self.config.iface_cache
-		try:
-			for item in get_inputs():
-				if not item: continue
-				if isinstance(item, tuple):
-					path, mtime = item
-				else:
-					path = item
-					try:
-						mtime = os.stat(path).st_mtime
-					except OSError as ex:
-						logger.info("Triggering update to {app} due to error: {ex}".format(
-							app = self, path = path, ex = ex))
+			# If any of the feeds we used have been updated since the last check, do a quick re-solve
+			try:
+				for item in get_inputs():
+					if not item: continue
+					if isinstance(item, tuple):
+						path, mtime = item
+					else:
+						path = item
+						try:
+							mtime = os.stat(path).st_mtime
+						except OSError as ex:
+							logger.info("Triggering update to {app} due to error: {ex}".format(
+								app = self, path = path, ex = ex))
+							need_solve = True
+							break
+
+					if mtime and mtime > last_solve:
+						logger.info("Triggering update to %s because %s has changed", self, path)
 						need_solve = True
 						break
-
-				if mtime and mtime > last_solve:
-					logger.info("Triggering update to %s because %s has changed", self, path)
-					need_solve = True
-					break
-		except Exception as ex:
-			logger.info("Error checking modification times: %s", ex)
-			need_solve = True
-			need_update = True
-
-		# Is it time for a background update anyway?
-		if not need_update:
-			staleness = time.time() - utime
-			logger.info("Staleness of app %s is %d hours", self, staleness / (60 * 60))
-			freshness_threshold = self.config.freshness
-			if freshness_threshold > 0 and staleness >= freshness_threshold:
+			except Exception as ex:
+				logger.info("Error checking modification times: %s", ex)
+				need_solve = True
 				need_update = True
 
-		# If any of the saved selections aren't available then we need
-		# to download right now, not later in the background.
-		unavailable_selections = sels.get_unavailable_selections(config = self.config, include_packages = True)
-		if unavailable_selections:
-			logger.info("Saved selections are unusable (missing %s)",
-				    ', '.join(str(s) for s in unavailable_selections))
+			# Is it time for a background update anyway?
+			if not need_update:
+				staleness = time.time() - utime
+				logger.info("Staleness of app %s is %d hours", self, staleness / (60 * 60))
+				freshness_threshold = self.config.freshness
+				if freshness_threshold > 0 and staleness >= freshness_threshold:
+					need_update = True
+
+			# If any of the saved selections aren't available then we need
+			# to download right now, not later in the background.
+			unavailable_selections = sels.get_unavailable_selections(config = self.config, include_packages = True)
+			if unavailable_selections:
+				logger.info("Saved selections are unusable (missing %s)",
+					    ', '.join(str(s) for s in unavailable_selections))
+				need_solve = True
+		else:
+			# No current selections
 			need_solve = True
+			unavailable_selections = True
 
 		if need_solve:
 			from zeroinstall.injector.driver import Driver
@@ -267,9 +299,12 @@ class App:
 				old_sels = sels
 				sels = driver.solver.selections
 				from zeroinstall.support import xmltools
-				if not xmltools.nodes_equal(sels.toDOM(), old_sels.toDOM()):
+				if old_sels is None or not xmltools.nodes_equal(sels.toDOM(), old_sels.toDOM()):
 					self.set_selections(sels, set_last_checked = False)
-			self._touch('last-solve')
+			try:
+				self._touch('last-solve')
+			except OSError as ex:
+				logger.warning("Error checking for updates: %s", ex)
 
 		# If we tried to check within the last hour, don't try again.
 		if need_update:
@@ -279,16 +314,22 @@ class App:
 				need_update = False
 
 		if need_update:
-			self.set_last_check_attempt()
-			from zeroinstall.injector import background
-			r = self.get_requirements()
-			background.spawn_background_update2(r, False, self)
+			try:
+				self.set_last_check_attempt()
+			except OSError as ex:
+				logger.warning("Error checking for updates: %s", ex)
+			else:
+				from zeroinstall.injector import background
+				r = self.get_requirements()
+				background.spawn_background_update2(r, False, self)
 
 		return sels
 
 	def _foreground_update(self, driver, use_gui):
 		"""We can't run with saved selections or solved selections without downloading.
-		Try to open the GUI for a blocking download. If we can't do that, download without the GUI."""
+		Try to open the GUI for a blocking download. If we can't do that, download without the GUI.
+		@type driver: L{zeroinstall.injector.driver.Driver}
+		@rtype: L{zeroinstall.injector.selections.Selections}"""
 		from zeroinstall import helpers
 		from zeroinstall.support import tasks
 
@@ -308,6 +349,7 @@ class App:
 		return sels
 
 	def set_requirements(self, requirements):
+		"""@type requirements: L{zeroinstall.injector.requirements.Requirements}"""
 		reqs_file = os.path.join(self.path, 'requirements.json')
 		if self.config.handler.dry_run:
 			print(_("[dry-run] would write {file}").format(file = reqs_file))
@@ -325,6 +367,7 @@ class App:
 			portable_rename(tmp.name, reqs_file)
 
 	def get_requirements(self):
+		"""@rtype: L{zeroinstall.injector.requirements.Requirements}"""
 		import json
 		from zeroinstall.injector import requirements
 		r = requirements.Requirements(None)
@@ -353,6 +396,7 @@ class App:
 		self._touch('last-checked')
 
 	def _touch(self, name):
+		"""@type name: str"""
 		timestamp_path = os.path.join(self.path, name)
 		if self.config.handler.dry_run:
 			pass #print(_("[dry-run] would update timestamp file {file}").format(file = timestamp_path))
@@ -362,6 +406,9 @@ class App:
 			os.utime(timestamp_path, None)	# In case file already exists
 
 	def _get_mtime(self, name, warn_if_missing = True):
+		"""@type name: str
+		@type warn_if_missing: bool
+		@rtype: int"""
 		timestamp_path = os.path.join(self.path, name)
 		try:
 			return os.stat(timestamp_path).st_mtime
@@ -413,6 +460,7 @@ class App:
 
 	def integrate_shell(self, name):
 		# TODO: remember which commands we create
+		"""@type name: str"""
 		if not valid_name.match(name):
 			raise SafeException("Invalid shell command name '{name}'".format(name = name))
 		bin_dir = find_bin_dir()
@@ -429,16 +477,22 @@ class App:
 				os.chmod(launcher, 0o111 | os.fstat(stream.fileno()).st_mode)
 
 	def get_name(self):
+		"""@rtype: str"""
 		return os.path.basename(self.path)
 
 	def __str__(self):
+		"""@rtype: str"""
 		return '<app ' + self.get_name() + '>'
 
-class AppManager:
+class AppManager(object):
 	def __init__(self, config):
+		"""@type config: L{zeroinstall.injector.config.Config}"""
 		self.config = config
 
 	def create_app(self, name, requirements):
+		"""@type name: str
+		@type requirements: L{zeroinstall.injector.requirements.Requirements}
+		@rtype: L{App}"""
 		validate_name(name)
 
 		apps_dir = basedir.save_config_path(namespaces.config_site, "apps")
@@ -461,7 +515,10 @@ class AppManager:
 		"""Get the App for name.
 		Returns None if name is not an application (doesn't exist or is not a valid name).
 		Since / and : are not valid name characters, it is generally safe to try this
-		before calling L{injector.model.canonical_iface_uri}."""
+		before calling L{injector.model.canonical_iface_uri}.
+		@type name: str
+		@type missing_ok: bool
+		@rtype: L{App}"""
 		if not valid_name.match(name):
 			if missing_ok:
 				return None
