@@ -15,13 +15,13 @@ import logging
 from zeroinstall import SafeException, DryRun
 
 valid_commands = ['add', 'select', 'show', 'download', 'run', 'update', 'whatchanged', 'destroy',
-		  'config', 'import', 'list', 'add-feed', 'remove-feed', 'list-feeds',
+		  'config', 'import', 'list', 'search', 'add-feed', 'remove-feed', 'list-feeds',
 		  'man', 'digest']
 
 class UsageError(Exception): pass
 
 def _ensure_standard_fds():
-	"Ensure stdin, stdout and stderr FDs exist, to avoid confusion."
+	"""Ensure stdin, stdout and stderr FDs exist, to avoid confusion."""
 	for std in (0, 1, 2):
 		try:
 			os.fstat(std)
@@ -31,7 +31,7 @@ def _ensure_standard_fds():
 				os.dup2(fd, std)
 				os.close(fd)
 
-class NoCommand:
+class NoCommand(object):
 	"""Handle --help and --version"""
 
 	def add_options(self, parser):
@@ -50,9 +50,11 @@ class NoCommand:
 			sys.exit(0)
 		raise UsageError()
 
-class _Completion():
+class _Completion(object):
 	def __init__(self, config, command_args, shell):
-		assert shell in ('zsh', 'bash'), shell
+		"""@type command_args: [str]
+		@type shell: str"""
+		assert shell in ('zsh', 'bash', 'fish'), shell
 		self.shell = shell
 		self.config = config
 		self.cword = int(os.environ['COMP_CWORD']) - 1
@@ -81,14 +83,13 @@ class _Completion():
 			elif command_args[self.cword] == '=':
 				command_args[self.cword] = ''
 			#print(command_args, self.cword, file = sys.stderr)
-			self.command_args = command_args
 
 		if self.cword < len(command_args):
 			self.current = command_args[self.cword]
 		else:
 			self.current = ''
 
-		if shell == 'zsh':
+		if shell in ('zsh', 'fish'):
 			if self.current.startswith('--') and '=' in self.current:
 				# Split "--foo=bar" into "--foo", "bar"
 				name, value = self.current.split('=', 1)
@@ -103,6 +104,8 @@ class _Completion():
 
 	def got_command(self, command, pos):
 		#print("found %s at %s [cword = %d]" % (command, pos, self.cword), file = sys.stderr)
+		"""@type command: str
+		@type pos: int"""
 		if pos == self.cword:
 			for command in valid_commands:
 				self.add("filter", command)
@@ -215,6 +218,9 @@ class _Completion():
 				self.add("add", self.current)
 
 	def expand_range(self, uri, maybe_app = False, range_ok = True):
+		"""@type uri: str
+		@type maybe_app: bool
+		@type range_ok: bool"""
 		if maybe_app:
 			app = self.config.app_mgr.lookup_app(uri, missing_ok = True)
 			if app:
@@ -264,14 +270,17 @@ class _Completion():
 			self.expand_files()
 
 	def add_filtered(self, value):
-		"""Add this value, but only if it matches the prefix."""
+		"""Add this value, but only if it matches the prefix.
+		@type value: str"""
 		self.add("filter", value)
 
 	def add(self, type, value):
 		"""Types are:
 		add - a raw string to add
 		filter - a string to add only if it matches
-		prefix - a completion that doesn't insert a space after it."""
+		prefix - a completion that doesn't insert a space after it.
+		@type type: str
+		@type value: str"""
 		if self.shell == 'bash':
 			if ':' in self.current:
 				ignored = self.current.rsplit(':', 1)[0] + ':'
@@ -284,9 +293,9 @@ class _Completion():
 
 def main(command_args, config = None):
 	"""Act as if 0install was run with the given arguments.
-	@arg command_args: array of arguments (e.g. C{sys.argv[1:]})
 	@type command_args: [str]
-	"""
+	@type config: L{zeroinstall.injector.config.Config} | None
+	@arg command_args: array of arguments (e.g. C{sys.argv[1:]})"""
 	_ensure_standard_fds()
 
 	if config is None:
@@ -318,6 +327,8 @@ def main(command_args, config = None):
 	verbose = False
 	try:
 		# Configure a parser for the given command
+		my_name = os.path.basename(sys.argv[0])
+		if my_name == '0launch': my_name = '0install'	# Hack for --python-fallback
 		if command:
 			if command not in valid_commands:
 				if completion:
@@ -326,11 +337,11 @@ def main(command_args, config = None):
 
 			module_name = command.replace('-', '_')
 			cmd = __import__('zeroinstall.cmd.' + module_name, globals(), locals(), [module_name], 0)
-			parser = OptionParser(usage=_("usage: %%prog %s [OPTIONS] %s") % (command, cmd.syntax))
+			parser = OptionParser(usage=_("usage: %s %s [OPTIONS] %s") % (my_name, command, cmd.syntax))
 		else:
 			cmd = NoCommand()
-			parser = OptionParser(usage=_("usage: %prog COMMAND\n\nTry --help with one of these:") +
-					"\n\n0install " + '\n0install '.join(valid_commands))
+			parser = OptionParser(usage=_("usage: %s COMMAND\n\nTry --help with one of these:%s") %
+					(my_name, "\n\n0install " + '\n0install '.join(valid_commands)))
 
 		parser.add_option("-c", "--console", help=_("never use GUI"), action='store_false', dest='gui')
 		parser.add_option("", "--dry-run", help=_("just print what would be executed"), action='store_true')
