@@ -9,6 +9,7 @@ from __future__ import print_function
 
 import sys
 
+from zeroinstall import _
 from zeroinstall.cmd import UsageError, select
 from zeroinstall.injector import model, requirements
 
@@ -18,6 +19,7 @@ def add_options(parser):
 	select.add_options(parser)
 
 def handle(config, options, args):
+	"""@type args: [str]"""
 	if len(args) != 2:
 		raise UsageError()
 
@@ -28,6 +30,12 @@ def handle(config, options, args):
 	if not sels:
 		sys.exit(1)	# Aborted by user
 
+	root_feed = config.iface_cache.get_feed(iface_uri)
+	if root_feed:
+		target = root_feed.get_replaced_by()
+		if target is not None:
+			print(_("Warning: interface {old} has been replaced by {new}".format(old = iface_uri, new = target)))
+
 	r = requirements.Requirements(iface_uri)
 	r.parse_options(options)
 
@@ -36,5 +44,8 @@ def handle(config, options, args):
 	app.integrate_shell(pet_name)
 
 def complete(completion, args, cword):
+	"""@type completion: L{zeroinstall.cmd._Completion}
+	@type args: [str]
+	@type cword: int"""
 	if cword != 1: return
 	completion.expand_interfaces()
