@@ -8,7 +8,6 @@ The B{0install run} command-line interface.
 import sys
 
 from zeroinstall import _
-from zeroinstall.support import tasks
 from zeroinstall.cmd import UsageError, select
 from zeroinstall.injector import model
 
@@ -21,6 +20,8 @@ def add_options(parser):
 	parser.disable_interspersed_args()
 
 def handle(config, options, args):
+	"""@type config: L{zeroinstall.injector.config.Config}
+	@type args: [str]"""
 	if len(args) < 1:
 		raise UsageError()
 
@@ -36,16 +37,18 @@ def handle(config, options, args):
 	if app is not None:
 		sels = app.get_selections(may_update = True, use_gui = options.gui)
 		r = app.get_requirements()
-		do_select = r.parse_update_options(options)
+		do_select = r.parse_update_options(options) or options.refresh
 		iface_uri = sels.interface
 	else:
 		iface_uri = model.canonical_iface_uri(args[0])
+		r = None
 		do_select = True
 
 	if do_select or options.gui:
 		sels = select.get_selections(config, options, iface_uri,
 					select_only = False, download_only = False,
-					test_callback = test_callback)
+					test_callback = test_callback,
+					requirements = r)
 		if not sels:
 			sys.exit(1)	# Aborted by user
 
@@ -53,6 +56,9 @@ def handle(config, options, args):
 	run.execute_selections(sels, prog_args, dry_run = options.dry_run, main = options.main, wrapper = options.wrapper, stores = config.stores)
 
 def complete(completion, args, cword):
+	"""@type completion: L{zeroinstall.cmd._Completion}
+	@type args: [str]
+	@type cword: int"""
 	if cword == 0:
 		select.complete(completion, args, cword)
 	else:
