@@ -10,6 +10,7 @@ from zeroinstall import _, logger
 import os, platform, re, subprocess, sys
 from zeroinstall.injector import namespaces, model, arch, qdom
 from zeroinstall.support import basedir, portable_rename, intern
+from zeroinstall.support.tasks import get_loop
 
 _dotted_ints = '[0-9]+(?:\.[0-9]+)*'
 
@@ -22,13 +23,21 @@ _version_regexp = '(?:[a-z])?({ints}\.?[bu])?({zero})(-r{ints})?'.format(zero =
 
 _PYTHON_URI = 'http://repo.roscidus.com/python/python'
 
+def _set_quick_test(impl, path):
+	"""Set impl.quick_test_file and impl.quick_test_mtime from path."""
+	impl.quick_test_file = path
+	impl.quick_test_mtime = int(os.stat(path).st_mtime)
+
 # We try to do updates atomically without locking, but we don't worry too much about
 # duplicate entries or being a little out of sync with the on-disk copy.
 class Cache(object):
 	def __init__(self, cache_leaf, source, format):
 		"""Maintain a cache file (e.g. ~/.cache/0install.net/injector/$name).
-		If the size or mtime of $source has changed, or the cache
-		format version if different, reset the cache first."""
+		If the size or mtime of $source has changed
+		format version if different, reset the cache first.
+		@type cache_leaf: str
+		@type source: str
+		@type format: int"""
 		self.cache_leaf = cache_leaf
 		self.source = source
 		self.format = format
@@ -89,6 +98,8 @@ class Cache(object):
 			raise Exception("Format of cache has changed")
 
 	def get(self, key):
+		"""@type key: str
+		@rtype: str"""
 		try:
 			self._check_valid()
 		except Exception as ex:
@@ -99,6 +110,8 @@ class Cache(object):
 			return self.cache.get(key, None)
 
 	def put(self, key, value):
+		"""@type key: str
+		@type value: str"""
 		cache_path = os.path.join(self.cache_dir, self.cache_leaf)
 		self.cache[key] = value
 		try:
@@ -110,6 +123,7 @@ class Cache(object):
 def try_cleanup_distro_version(version):
 	"""Try to turn a distribution version string into one readable by Zero Install.
 	We do this by stripping off anything we can't parse.
+	@type version: str
 	@return: the part we understood, or None if we couldn't parse anything
 	@rtype: str"""
 	if ':' in version:
@@ -139,12 +153,16 @@ class Distribution(object):
 	@since: 0.28
 	@ivar name: the default value for Implementation.distro_name for our implementations
 	@type name: str
+	@ivar system_paths: list of paths to search for binaries (we MUST NOT find 0install launchers, so only include directories where system packages install binaries - e.g. /usr/bin but not /usr/local/bin)
+	@type system_paths: [str]
 	"""
 
 	name = "fallback"
 
 	_packagekit = None
 
+	system_paths = ['/usr/bin', '/bin', '/usr/sbin', '/sbin']
+
 	def get_package_info(self, package, factory):
 		"""Get information about the given package.
 		Add zero or more implementations using the factory (typically at most two
@@ -152,8 +170,7 @@ class Distribution(object):
 		@param package: package name (e.g. "gimp")
 		@type package: str
 		@param factory: function for creating new DistributionImplementation objects from IDs
-		@type factory: str -> L{model.DistributionImplementation}
-		"""
+		@type factory: str -> L{model.DistributionImplementation}"""
 		return
 
 	def get_score(self, distribution):
@@ -164,8 +181,7 @@ class Distribution(object):
 		@param distribution: a distribution name
 		@type distribution: str
 		@return: an integer, or -1 if there is no match at all
-		@rtype: int
-		"""
+		@rtype: int"""
 		return 0
 
 	def get_feed(self, master_feed):
@@ -205,11 +221,7 @@ class Distribution(object):
 				if 'run' not in impl.commands:
 					item_main = item_attrs.get('main', None)
 					if item_main:
-						if item_main.startswith('/'):
-							impl.main = item_main
-						else:
-							raise model.InvalidInterface(_("'main' attribute must be absolute, but '%s' doesn't start with '/'!") %
-										item_main)
+						impl.main = item_main
 				impl.upstream_stability = model.packaged
 
 				return impl
@@ -233,23 +245,32 @@ class Distribution(object):
 			impl.main = sys.executable or '/usr/bin/python'
 			impl.upstream_stability = model.packaged
 			impl.machine = host_machine	# (hopefully)
+
+			_set_quick_test(impl, sys.executable)
+
 			feed.implementations[impl_id] = impl
 		elif master_feed.url == 'http://repo.roscidus.com/python/python-gobject' and os.name != "nt":
-			# Likewise, we know that there is a native python-gobject available for our Python
-			from zeroinstall import gobject
-			impl_id = 'package:host:python-gobject:' + '.'.join(str(x) for x in gobject.pygobject_version)
-			assert impl_id not in feed.implementations
-			impl = model.DistributionImplementation(feed, impl_id, self, distro_name = 'host')
-			impl.installed = True
-			impl.version = [list(gobject.pygobject_version)]
-			impl.upstream_stability = model.packaged
-			impl.machine = host_machine	# (hopefully)
+			gobject = get_loop().gobject
+			if gobject:
+				# Likewise, we know that there is a native python-gobject available for our Python
+				impl_id = 'package:host:python-gobject:' + '.'.join(str(x) for x in gobject.pygobject_version)
+				assert impl_id not in feed.implementations
+				impl = model.DistributionImplementation(feed, impl_id, self, distro_name = 'host')
+				impl.installed = True
+				impl.version = [list(gobject.pygobject_version)]
+				impl.upstream_stability = model.packaged
+				impl.machine = host_machine	# (hopefully)
 
-			# Requires our version of Python too
-			restriction_element = qdom.Element(namespaces.XMLNS_IFACE, 'restricts', {'interface': _PYTHON_URI, 'distribution': 'host'})
-			impl.requires.append(model.process_depends(restriction_element, None))
+				if gobject.__file__.startswith('<'):
+					_set_quick_test(impl, gobject.__path__)		# Python 3
+				else:
+					_set_quick_test(impl, gobject.__file__)		# Python 2
 
-			feed.implementations[impl_id] = impl
+				# Requires our version of Python too
+				restriction_element = qdom.Element(namespaces.XMLNS_IFACE, 'restricts', {'interface': _PYTHON_URI, 'distribution': 'host'})
+				impl.requires.append(model.process_depends(restriction_element, None))
+
+				feed.implementations[impl_id] = impl
 
 		return feed
 
@@ -282,16 +303,44 @@ class Distribution(object):
 	def installed_fixup(self, impl):
 		"""Called when an installed package is added (after L{fixup}), or when installation
 		completes. This is useful to fix up the main value.
+		The default implementation checks that main exists, and searches L{Distribution.system_paths} for
+		it if not.
 		@type impl: L{DistributionImplementation}
 		@since: 1.11"""
-		pass
+
+		run = impl.commands.get('run', None)
+		if not run: return
+
+		path = run.path
+
+		if not path: return
+
+		if os.path.isabs(path) and os.path.exists(path):
+			return
+
+		basename = os.path.basename(path)
+		if os.name == "nt" and not basename.endswith('.exe'):
+			basename += '.exe'
+
+		for d in self.system_paths:
+			path = os.path.join(d, basename)
+			if os.path.isfile(path):
+				logger.info("Found %s by searching system paths", path)
+				run.qdom.attrs["path"] = path
+				return
+		else:
+			logger.info("Binary '%s' not found in any system path (checked %s)", basename, self.system_paths)
 
 	def get_score(self, distro_name):
+		"""@type distro_name: str
+		@rtype: int"""
 		return int(distro_name == self.name)
 
 class WindowsDistribution(Distribution):
 	name = 'Windows'
 
+	system_paths = []
+
 	def get_package_info(self, package, factory):
 		def _is_64bit_windows():
 			p = sys.platform
@@ -339,6 +388,7 @@ class WindowsDistribution(Distribution):
 					impl.version = model.parse_version(zero_version)
 					impl.upstream_stability = model.packaged
 					impl.main = home + r"\bin\java.exe"
+					_set_quick_test(impl, impl.main)
 
 		def find_netfx(win_version, zero_version):
 			reg_path = r"SOFTWARE\Microsoft\NET Framework Setup\NDP\{win_version}".format(win_version = win_version)
@@ -390,6 +440,7 @@ class DarwinDistribution(Distribution):
 	name = 'Darwin'
 
 	def get_package_info(self, package, factory):
+		"""@type package: str"""
 		def java_home(version, arch):
 			null = os.open(os.devnull, os.O_WRONLY)
 			child = subprocess.Popen(["/usr/libexec/java_home", "--failfast", "--version", version, "--arch", arch],
@@ -408,6 +459,7 @@ class DarwinDistribution(Distribution):
 					impl.version = model.parse_version(zero_version)
 					impl.upstream_stability = model.packaged
 					impl.main = home + "/bin/java"
+					_set_quick_test(impl, impl.main)
 
 		if package == 'openjdk-6-jre':
 			find_java("Java Runtime Environment", "1.6", '6')
@@ -436,6 +488,7 @@ class DarwinDistribution(Distribution):
 					impl.upstream_stability = model.packaged
 					impl.machine = host_machine	# (hopefully)
 					impl.main = file
+					_set_quick_test(impl, impl.main)
 
 		if package == 'gnupg':
 			find_program("/usr/local/bin/gpg")
@@ -450,7 +503,8 @@ class CachedDistribution(Distribution):
 	"""
 
 	def __init__(self, db_status_file):
-		"""@param db_status_file: update the cache when the timestamp of this file changes"""
+		"""@param db_status_file: update the cache when the timestamp of this file changes
+		@type db_status_file: str"""
 		self._status_details = os.stat(db_status_file)
 
 		self.versions = {}
@@ -499,6 +553,7 @@ class CachedDistribution(Distribution):
 
 	def _write_cache(self, cache):
 		#cache.sort() 	# Might be useful later; currently we don't care
+		"""@type cache: [str]"""
 		import tempfile
 		fd, tmpname = tempfile.mkstemp(prefix = 'zeroinstall-cache-tmp',
 					       dir = self.cache_dir)
@@ -538,6 +593,8 @@ _canonical_machine = {
 
 host_machine = arch.canonicalize_machine(platform.uname()[4])
 def canonical_machine(package_machine):
+	"""@type package_machine: str
+	@rtype: str"""
 	machine = _canonical_machine.get(package_machine.lower(), None)
 	if machine is None:
 		# Safe default if we can't understand the arch
@@ -552,10 +609,13 @@ class DebianDistribution(Distribution):
 	cache_leaf = 'dpkg-status.cache'
 
 	def __init__(self, dpkg_status):
+		"""@type dpkg_status: str"""
 		self.dpkg_cache = Cache('dpkg-status.cache', dpkg_status, 2)
 		self.apt_cache = {}
 
 	def _query_installed_package(self, package):
+		"""@type package: str
+		@rtype: str"""
 		null = os.open(os.devnull, os.O_WRONLY)
 		child = subprocess.Popen(["dpkg-query", "-W", "--showformat=${Version}\t${Architecture}\t${Status}\n", "--", package],
 						stdout = subprocess.PIPE, stderr = null,
@@ -579,6 +639,7 @@ class DebianDistribution(Distribution):
 
 	def get_package_info(self, package, factory):
 		# Add any already-installed package...
+		"""@type package: str"""
 		installed_cached_info = self._get_dpkg_info(package)
 
 		if installed_cached_info != '-':
@@ -610,8 +671,10 @@ class DebianDistribution(Distribution):
 							"Please install it manually using your distribution's tools and try again. Or, install 'packagekit' and I can "
 							"use that to install it.") % package)
 				impl.download_sources.append(model.DistributionSource(package, cached['size'], install, needs_confirmation = False))
-	
+
 	def fixup(self, package, impl):
+		"""@type package: str
+		@type impl: L{zeroinstall.injector.model.DistributionImplementation}"""
 		if impl.id.startswith('package:deb:openjdk-6-jre:') or \
 		   impl.id.startswith('package:deb:openjdk-7-jre:'):
 			# Debian marks all Java versions as pre-releases
@@ -619,13 +682,14 @@ class DebianDistribution(Distribution):
 			impl.version = model.parse_version(impl.get_version().replace('-pre', '.'))
 
 	def installed_fixup(self, impl):
+		"""@type impl: L{zeroinstall.injector.model.DistributionImplementation}"""
 		# Hack: If we added any Java implementations, find the corresponding JAVA_HOME...
 		if impl.id.startswith('package:deb:openjdk-6-jre:'):
 			java_version = '6-openjdk'
 		elif impl.id.startswith('package:deb:openjdk-7-jre:'):
 			java_version = '7-openjdk'
 		else:
-			return
+			return Distribution.installed_fixup(self, impl)		# super
 
 		if impl.machine == 'x86_64':
 			java_arch = 'amd64'
@@ -647,6 +711,8 @@ class DebianDistribution(Distribution):
 			{'path': java_bin, 'name': 'run'}), None)
 
 	def _get_dpkg_info(self, package):
+		"""@type package: str
+		@rtype: str"""
 		installed_cached_info = self.dpkg_cache.get(package)
 		if installed_cached_info == None:
 			installed_cached_info = self._query_installed_package(package)
@@ -655,6 +721,8 @@ class DebianDistribution(Distribution):
 		return installed_cached_info
 
 	def fetch_candidates(self, master_feed):
+		"""@type master_feed: L{zeroinstall.injector.model.ZeroInstallFeed}
+		@rtype: [L{zeroinstall.support.tasks.Blocker}]"""
 		package_names = [item.getAttribute("package") for item, item_attrs, depends in master_feed.get_package_impls(self)]
 
 		if self.packagekit.available:
@@ -719,6 +787,7 @@ class RPMDistribution(CachedDistribution):
 
 	def get_package_info(self, package, factory):
 		# Add installed versions...
+		"""@type package: str"""
 		versions = self.versions.get(package, [])
 
 		for version, machine in versions:
@@ -732,6 +801,7 @@ class RPMDistribution(CachedDistribution):
 
 	def installed_fixup(self, impl):
 		# OpenSUSE uses _, Fedora uses .
+		"""@type impl: L{zeroinstall.injector.model.DistributionImplementation}"""
 		impl_id = impl.id.replace('_', '.')
 
 		# Hack: If we added any Java implementations, find the corresponding JAVA_HOME...
@@ -741,7 +811,7 @@ class RPMDistribution(CachedDistribution):
 		elif impl_id.startswith('package:rpm:java-1.7.0-openjdk:'):
 			java_version = '1.7.0-openjdk'
 		else:
-			return
+			return Distribution.installed_fixup(self, impl)		# super
 
 		# On Fedora, unlike Debian, the arch is x86_64, not amd64
 
@@ -761,6 +831,8 @@ class RPMDistribution(CachedDistribution):
 
 	def fixup(self, package, impl):
 		# OpenSUSE uses _, Fedora uses .
+		"""@type package: str
+		@type impl: L{zeroinstall.injector.model.DistributionImplementation}"""
 		package = package.replace('_', '.')
 
 		if package in ('java-1.6.0-openjdk', 'java-1.7.0-openjdk',
@@ -775,10 +847,12 @@ class SlackDistribution(Distribution):
 	name = 'Slack'
 
 	def __init__(self, packages_dir):
+		"""@type packages_dir: str"""
 		self._packages_dir = packages_dir
 
 	def get_package_info(self, package, factory):
 		# Add installed versions...
+		"""@type package: str"""
 		for entry in os.listdir(self._packages_dir):
 			name, version, arch, build = entry.rsplit('-', 3)
 			if name == package:
@@ -803,21 +877,24 @@ class ArchDistribution(Distribution):
 	name = 'Arch'
 
 	def __init__(self, packages_dir):
+		"""@type packages_dir: str"""
 		self._packages_dir = os.path.join(packages_dir, "local")
 
 	def get_package_info(self, package, factory):
 		# Add installed versions...
+		"""@type package: str"""
 		for entry in os.listdir(self._packages_dir):
 			name, version, build = entry.rsplit('-', 2)
 			if name == package:
 				gotarch = False
-				with open(os.path.join(self._packages_dir, entry, "desc"), 'rt') as stream:
+				# (read in binary mode to avoid unicode errors in C locale)
+				with open(os.path.join(self._packages_dir, entry, "desc"), 'rb') as stream:
 					for line in stream:
-						if line == "%ARCH%\n":
+						if line == b"%ARCH%\n":
 							gotarch = True
 							continue
 						if gotarch:
-							arch = line.strip()
+							arch = line.strip().decode('utf-8')
 							break
 				zi_arch = canonical_machine(arch)
 				clean_version = try_cleanup_distro_version("%s-%s" % (version, build))
@@ -831,6 +908,8 @@ class ArchDistribution(Distribution):
 				if zi_arch != '*':
 					impl.machine = zi_arch
 
+				impl.quick_test_file = os.path.join(self._packages_dir, entry, 'desc')
+
 		# Add any uninstalled candidates found by PackageKit
 		self.packagekit.get_candidates(package, factory, 'package:arch')
 
@@ -838,10 +917,12 @@ class GentooDistribution(Distribution):
 	name = 'Gentoo'
 
 	def __init__(self, pkgdir):
+		"""@type pkgdir: str"""
 		self._pkgdir = pkgdir
 
 	def get_package_info(self, package, factory):
 		# Add installed versions...
+		"""@type package: str"""
 		_version_start_reqexp = '-[0-9]'
 
 		if package.count('/') != 1: return
@@ -882,10 +963,14 @@ class GentooDistribution(Distribution):
 class PortsDistribution(Distribution):
 	name = 'Ports'
 
+	system_paths = ['/usr/local/bin']
+
 	def __init__(self, pkgdir):
+		"""@type pkgdir: str"""
 		self._pkgdir = pkgdir
 
 	def get_package_info(self, package, factory):
+		"""@type package: str"""
 		_name_version_regexp = '^(.+)-([^-]+)$'
 
 		nameversion = re.compile(_name_version_regexp)
@@ -913,9 +998,12 @@ class PortsDistribution(Distribution):
 			impl.machine = machine
 
 class MacPortsDistribution(CachedDistribution):
+	system_paths = ['/opt/local/bin']
+
 	name = 'MacPorts'
 
 	def __init__(self, db_status_file):
+		"""@type db_status_file: str"""
 		super(MacPortsDistribution, self).__init__(db_status_file)
 		self.darwin = DarwinDistribution()
 
@@ -956,6 +1044,7 @@ class MacPortsDistribution(CachedDistribution):
 		child.wait()
 
 	def get_package_info(self, package, factory):
+		"""@type package: str"""
 		self.darwin.get_package_info(package, factory)
 
 		# Add installed versions...
@@ -984,7 +1073,7 @@ class CygwinDistribution(CachedDistribution):
 	def generate_cache(self):
 		cache = []
 
-		zi_arch = canonical_machine(arch)
+		zi_arch = '*'
 		for line in os.popen("cygcheck -c -d"):
 			if line == "Cygwin Package Information\r\n":
 				continue
