@@ -46,6 +46,7 @@ class Handler(object):
 	__slots__ = ['monitored_downloads', 'dry_run', 'total_bytes_downloaded', 'n_completed_downloads']
 
 	def __init__(self, mainloop = None, dry_run = False):
+		"""@type dry_run: bool"""
 		self.monitored_downloads = set()
 		self.dry_run = dry_run
 		self.n_completed_downloads = 0
@@ -53,7 +54,8 @@ class Handler(object):
 
 	def monitor_download(self, dl):
 		"""Called when a new L{download} is started.
-		This is mainly used by the GUI to display the progress bar."""
+		This is mainly used by the GUI to display the progress bar.
+		@type dl: L{zeroinstall.injector.download.Download}"""
 		self.monitored_downloads.add(dl)
 		self.downloads_changed()
 
@@ -74,8 +76,7 @@ class Handler(object):
 		"""Called by the L{fetch.Fetcher} when adding an implementation.
 		The GUI uses this to update its display.
 		@param impl: the implementation which has been added
-		@type impl: L{model.Implementation}
-		"""
+		@type impl: L{model.Implementation}"""
 		pass
 	
 	def downloads_changed(self):
@@ -83,7 +84,8 @@ class Handler(object):
 		pass
 	
 	def wait_for_blocker(self, blocker):
-		"""@deprecated: use tasks.wait_for_blocker instead"""
+		"""@type blocker: L{zeroinstall.support.tasks.Blocker}
+		@deprecated: use tasks.wait_for_blocker instead"""
 		tasks.wait_for_blocker(blocker)
 	
 	@tasks.async
@@ -186,7 +188,7 @@ class Handler(object):
 		@since: 0.25"""
 		import logging
 		logger.warning("%s", str(exception) or type(exception),
-				exc_info = (exception, None, tb) if logger.isEnabledFor(logging.INFO) else None)
+				exc_info = (exception, exception, tb) if logger.isEnabledFor(logging.INFO) else None)
 	
 class ConsoleHandler(Handler):
 	"""A Handler that displays progress on stdout (a tty).
@@ -200,7 +202,6 @@ class ConsoleHandler(Handler):
 	original_print = None
 
 	def downloads_changed(self):
-		from zeroinstall import gobject
 		if self.monitored_downloads and self.update is None:
 			if self.screen_width is None:
 				try:
@@ -213,20 +214,20 @@ class ConsoleHandler(Handler):
 			self.show_progress()
 			self.original_print = print
 			builtins.print = self.print
-			self.update = gobject.timeout_add(200, self.show_progress)
+			self.update = tasks.get_loop().call_repeatedly(0.2, self.show_progress)
 		elif len(self.monitored_downloads) == 0:
 			if self.update:
-				gobject.source_remove(self.update)
+				self.update.cancel()
 				self.update = None
 				builtins.print = self.original_print
 				self.original_print = None
 				self.clear_display()
 
 	def show_progress(self):
-		if not self.monitored_downloads: return True
+		if not self.monitored_downloads: return
 		urls = [(dl.url, dl) for dl in self.monitored_downloads]
 
-		if self.disable_progress: return True
+		if self.disable_progress: return
 
 		screen_width = self.screen_width - 2
 		item_width = max(16, screen_width // len(self.monitored_downloads))
@@ -235,6 +236,8 @@ class ConsoleHandler(Handler):
 		msg = ""
 		for url, dl in sorted(urls):
 			so_far = dl.get_bytes_downloaded_so_far()
+			if url.endswith('/latest.xml'):
+				url = url[:-10]		# remove latest.xml from mirror URLs
 			leaf = url.rsplit('/', 1)[-1]
 			if len(leaf) >= url_width:
 				display = leaf[:url_width]
@@ -256,7 +259,7 @@ class ConsoleHandler(Handler):
 		self.last_msg_len = len(msg)
 		sys.stdout.flush()
 
-		return True
+		return
 
 	def clear_display(self):
 		if self.last_msg_len != None:
