@@ -23,7 +23,7 @@ from zeroinstall import support, zerostore
 from zeroinstall.support import escaping
 
 # Element names for bindings in feed files
-binding_names = frozenset(['environment', 'overlay', 'executable-in-path', 'executable-in-var'])
+binding_names = frozenset(['environment', 'overlay', 'executable-in-path', 'executable-in-var', 'binding'])
 
 _dependency_names = frozenset(['requires', 'restricts'])
 
@@ -45,6 +45,7 @@ class InvalidInterface(SafeException):
 	feed_url = None
 
 	def __init__(self, message, ex = None):
+		"""@type message: str"""
 		if ex:
 			try:
 				message += "\n\n(exact error: %s)" % ex
@@ -60,6 +61,7 @@ class InvalidInterface(SafeException):
 		SafeException.__init__(self, message)
 
 	def __unicode__(self):
+		"""@rtype: str"""
 		if hasattr(SafeException, '__unicode__'):
 			# Python >= 2.6
 			if self.feed_url:
@@ -69,7 +71,8 @@ class InvalidInterface(SafeException):
 			return support.unicode(SafeException.__str__(self))
 
 def _split_arch(arch):
-	"""Split an arch into an (os, machine) tuple. Either or both parts may be None."""
+	"""Split an arch into an (os, machine) tuple. Either or both parts may be None.
+	@type arch: str"""
 	if not arch:
 		return None, None
 	elif '-' not in arch:
@@ -81,10 +84,15 @@ def _split_arch(arch):
 		return osys, machine
 
 def _join_arch(osys, machine):
+	"""@type osys: str
+	@type machine: str
+	@rtype: str"""
 	if osys == machine == None: return None
 	return "%s-%s" % (osys or '*', machine or '*')
 
 def _best_language_match(options):
+	"""@type options: {str: str}
+	@rtype: str"""
 	(language, encoding) = locale.getlocale()
 
 	if language:
@@ -102,6 +110,9 @@ class Stability(object):
 	optionally, a user-set rating."""
 	__slots__ = ['level', 'name', 'description']
 	def __init__(self, level, name, description):
+		"""@type level: int
+		@type name: str
+		@type description: str"""
 		self.level = level
 		self.name = name
 		self.description = description
@@ -109,28 +120,37 @@ class Stability(object):
 		stability_levels[name] = self
 
 	def __cmp__(self, other):
+		"""@type other: L{Stability}
+		@rtype: int"""
 		return cmp(self.level, other.level)
 
 	def __lt__(self, other):
+		"""@type other: L{Stability}
+		@rtype: bool"""
 		if isinstance(other, Stability):
 			return self.level < other.level
 		else:
 			return NotImplemented
 
 	def __eq__(self, other):
+		"""@type other: L{Stability}
+		@rtype: bool"""
 		if isinstance(other, Stability):
 			return self.level == other.level
 		else:
 			return NotImplemented
 
 	def __str__(self):
+		"""@rtype: str"""
 		return self.name
 
 	def __repr__(self):
 		return _("<Stability: %s>") % self.description
 
 def process_binding(e):
-	"""Internal"""
+	"""Internal
+	@type e: L{zeroinstall.injector.qdom.Element}
+	@rtype: L{Binding}"""
 	if e.name == 'environment':
 		mode = {
 			None: EnvironmentBinding.PREPEND,
@@ -157,11 +177,16 @@ def process_binding(e):
 		return ExecutableBinding(e, in_path = False)
 	elif e.name == 'overlay':
 		return OverlayBinding(e.getAttribute('src'), e.getAttribute('mount-point'))
+	elif e.name == 'binding':
+		return GenericBinding(e)
 	else:
 		raise Exception(_("Unknown binding type '%s'") % e.name)
 
 def process_depends(item, local_feed_dir):
-	"""Internal"""
+	"""Internal
+	@type item: L{zeroinstall.injector.qdom.Element}
+	@type local_feed_dir: str
+	@rtype: L{Dependency}"""
 	# Note: also called from selections
 	# Note: used by 0compile
 	attrs = item.attrs
@@ -226,14 +251,14 @@ class Restriction(object):
 	def meets_restriction(self, impl):
 		"""Called by the L{solver.Solver} to check whether a particular implementation is acceptable.
 		@return: False if this implementation is not a possibility
-		@rtype: bool
-		"""
+		@rtype: bool"""
 		raise NotImplementedError(_("Abstract"))
 
 	def __str__(self):
 		return "missing __str__ on %s" % type(self)
 
 	def __repr__(self):
+		"""@rtype: str"""
 		return "<restriction: %s>" % self
 
 class VersionRestriction(Restriction):
@@ -242,12 +267,13 @@ class VersionRestriction(Restriction):
 
 	def __init__(self, version):
 		"""@param version: the required version number
-		@see: L{parse_version}; use this to pre-process the version number
-		"""
+		@see: L{parse_version}; use this to pre-process the version number"""
 		assert not isinstance(version, str), "Not parsed: " + version
 		self.version = version
 
 	def meets_restriction(self, impl):
+		"""@type impl: L{ZeroInstallImplementation}
+		@rtype: bool"""
 		return impl.version == self.version
 
 	def __str__(self):
@@ -260,12 +286,13 @@ class VersionRangeRestriction(Restriction):
 	def __init__(self, before, not_before):
 		"""@param before: chosen versions must be earlier than this
 		@param not_before: versions must be at least this high
-		@see: L{parse_version}; use this to pre-process the versions
-		"""
+		@see: L{parse_version}; use this to pre-process the versions"""
 		self.before = before
 		self.not_before = not_before
 
 	def meets_restriction(self, impl):
+		"""@type impl: L{Implementation}
+		@rtype: bool"""
 		if self.not_before and impl.version < self.not_before:
 			return False
 		if self.before and impl.version >= self.before:
@@ -273,6 +300,7 @@ class VersionRangeRestriction(Restriction):
 		return True
 
 	def __str__(self):
+		"""@rtype: str"""
 		if self.not_before is not None or self.before is not None:
 			range = ''
 			if self.not_before is not None:
@@ -297,9 +325,12 @@ class VersionExpressionRestriction(Restriction):
 		self._test_fn = versions.parse_version_expression(expr)
 
 	def meets_restriction(self, impl):
+		"""@type impl: L{Implementation}
+		@rtype: bool"""
 		return self._test_fn(impl.version)
 
 	def __str__(self):
+		"""@rtype: str"""
 		return "version " + self.expr
 
 class ImpossibleRestriction(Restriction):
@@ -308,12 +339,16 @@ class ImpossibleRestriction(Restriction):
 	@since: 1.13"""
 
 	def __init__(self, reason):
+		"""@type reason: str"""
 		self.reason = reason
 
 	def meets_restriction(self, impl):
+		"""@type impl: L{Implementation}
+		@rtype: bool"""
 		return False
 
 	def __str__(self):
+		"""@rtype: str"""
 		return "<impossible: %s>" % self.reason
 
 class DistributionRestriction(Restriction):
@@ -325,12 +360,16 @@ class DistributionRestriction(Restriction):
 	distros = None
 
 	def __init__(self, distros):
+		"""@type distros: str"""
 		self.distros = frozenset(distros.split(' '))
 
 	def meets_restriction(self, impl):
+		"""@type impl: L{Implementation}
+		@rtype: bool"""
 		return impl.distro_name in self.distros
 
 	def __str__(self):
+		"""@rtype: str"""
 		return "distro " + '|'.join(sorted(self.distros))
 
 class Binding(object):
@@ -389,7 +428,9 @@ class EnvironmentBinding(Binding):
 			return extra
 
 		if old_value is None:
-			old_value = self.default or defaults.get(self.name, None)
+			old_value = self.default
+			if old_value is None:
+				old_value = defaults.get(self.name, None)
 		if old_value is None:
 			return extra
 		if self.mode == EnvironmentBinding.PREPEND:
@@ -443,6 +484,24 @@ class ExecutableBinding(Binding):
 	def command(self):
 		return self.qdom.getAttribute("command") or 'run'
 
+class GenericBinding(Binding):
+	__slots__ = ['qdom']
+
+	def __init__(self, qdom):
+		self.qdom = qdom
+
+	def __str__(self):
+		return str(self.qdom)
+
+	__repr__ = __str__
+
+	def _toxml(self, doc, prefixes):
+		return self.qdom.toDOM(doc, prefixes)
+
+	@property
+	def command(self):
+		return self.qdom.getAttribute("command") or None
+
 class OverlayBinding(Binding):
 	"""Make the chosen implementation available by overlaying it onto another part of the file-system.
 	This is to support legacy programs which use hard-coded paths."""
@@ -503,6 +562,7 @@ class Dependency(object):
 	Restricts = "restricts"		# Just adds restrictions without expressing any opinion
 
 	def __init__(self, element):
+		"""@type element: L{zeroinstall.injector.qdom.Element}"""
 		assert isinstance(element, qdom.Element), type(element)	# Use InterfaceDependency instead!
 		self.qdom = element
 
@@ -525,6 +585,8 @@ class InterfaceRestriction(Dependency):
 	__slots__ = ['interface', 'restrictions']
 
 	def __init__(self, interface, restrictions = None, element = None):
+		"""@type interface: str
+		@type element: L{zeroinstall.injector.qdom.Element} | None"""
 		Dependency.__init__(self, element)
 		assert isinstance(interface, (str, support.unicode))
 		assert interface
@@ -553,10 +615,13 @@ class InterfaceDependency(InterfaceRestriction):
 	__slots__ = ['bindings']
 
 	def __init__(self, interface, restrictions = None, element = None):
+		"""@type interface: str
+		@type element: L{zeroinstall.injector.qdom.Element} | None"""
 		InterfaceRestriction.__init__(self, interface, restrictions, element)
 		self.bindings = []
 
 	def __str__(self):
+		"""@rtype: str"""
 		return _("<Dependency on %(interface)s; bindings: %(bindings)s%(restrictions)s>") % {'interface': self.interface, 'bindings': self.bindings, 'restrictions': self.restrictions}
 
 	@property
@@ -585,26 +650,60 @@ class RetrievalMethod(object):
 	"""A RetrievalMethod provides a way to fetch an implementation."""
 	__slots__ = []
 
+	requires_network = True		# Used to decide if we can get this in off-line mode
+
 class DownloadSource(RetrievalMethod):
 	"""A DownloadSource provides a way to fetch an implementation."""
-	__slots__ = ['implementation', 'url', 'size', 'extract', 'start_offset', 'type']
+	__slots__ = ['implementation', 'url', 'size', 'extract', 'start_offset', 'type', 'dest', 'requires_network']
 
-	def __init__(self, implementation, url, size, extract, start_offset = 0, type = None):
+	def __init__(self, implementation, url, size, extract, start_offset = 0, type = None, dest = None):
+		"""@type implementation: L{ZeroInstallImplementation}
+		@type url: str
+		@type size: int
+		@type extract: str
+		@type start_offset: int
+		@type type: str | None
+		@type dest: str | None"""
 		self.implementation = implementation
 		self.url = url
 		self.size = size
 		self.extract = extract
+		self.dest = dest
 		self.start_offset = start_offset
 		self.type = type		# MIME type - see unpack.py
+		self.requires_network = '://' in url
 
 class RenameStep(RetrievalMethod):
 	"""A Rename provides a way to rename / move a file within an implementation."""
 	__slots__ = ['source', 'dest']
 
 	def __init__(self, source, dest):
+		"""@type source: str
+		@type dest: str"""
 		self.source = source
 		self.dest = dest
 
+class FileSource(RetrievalMethod):
+	"""A FileSource provides a way to fetch a single file."""
+	__slots__ = ['url', 'dest', 'size', 'requires_network']
+
+	def __init__(self, url, dest, size):
+		"""@type url: str
+		@type dest: str
+		@type size: int"""
+		self.url = url
+		self.dest = dest
+		self.size = size
+		self.requires_network = '://' in url
+
+class RemoveStep(RetrievalMethod):
+	"""A RemoveStep provides a way to delete a path within an implementation."""
+	__slots__ = ['path']
+
+	def __init__(self, path):
+		"""@type path: str"""
+		self.path = path
+
 class Recipe(RetrievalMethod):
 	"""Get an implementation by following a series of steps.
 	@ivar size: the combined download sizes from all the steps
@@ -616,7 +715,11 @@ class Recipe(RetrievalMethod):
 	def __init__(self):
 		self.steps = []
 
-	size = property(lambda self: sum([x.size for x in self.steps if isinstance(x, DownloadSource)]))
+	size = property(lambda self: sum([x.size for x in self.steps if hasattr(x, 'size')]))
+
+	@property
+	def requires_network(self):
+		return any(step.requires_network for step in self.steps)
 
 class DistributionSource(RetrievalMethod):
 	"""A package that is installed using the distribution's tools (including PackageKit).
@@ -632,6 +735,9 @@ class DistributionSource(RetrievalMethod):
 	__slots__ = ['package_id', 'size', 'install', 'needs_confirmation']
 
 	def __init__(self, package_id, size, install, needs_confirmation = True):
+		"""@type package_id: str
+		@type size: int
+		@type needs_confirmation: bool"""
 		RetrievalMethod.__init__(self)
 		self.package_id = package_id
 		self.size = size
@@ -645,8 +751,8 @@ class Command(object):
 
 	def __init__(self, qdom, local_dir):
 		"""@param qdom: the <command> element
-		@param local_dir: the directory containing the feed (for relative dependencies), or None if not local
-		"""
+		@type qdom: L{zeroinstall.injector.qdom.Element}
+		@param local_dir: the directory containing the feed (for relative dependencies), or None if not local"""
 		assert qdom.name == 'command', 'not <command>: %s' % qdom
 		self.qdom = qdom
 		self._local_dir = local_dir
@@ -656,6 +762,7 @@ class Command(object):
 	path = property(lambda self: self.qdom.attrs.get("path", None))
 
 	def _toxml(self, doc, prefixes):
+		"""@type prefixes: L{zeroinstall.injector.qdom.Prefixes}"""
 		return self.qdom.toDOM(doc, prefixes)
 
 	@property
@@ -678,6 +785,7 @@ class Command(object):
 		return self._depends
 
 	def get_runner(self):
+		"""@rtype: L{InterfaceDependency}"""
 		self.requires		# (sets _runner)
 		return self._runner
 
@@ -723,6 +831,10 @@ class Implementation(object):
 	@type local_path: str | None
 	@ivar requires_root_install: whether the user will need admin rights to use this
 	@type requires_root_install: bool
+	@ivar quick_test_file: a file whose existence can be used later to check whether we need to update (since 2.2)
+	@type quick_test_file: str | None
+	@ivar quick_test_mtime: if present, requires that quick_test_file also has the given mtime
+	@type quick_test_mtime: int | None
 	"""
 
 	# Note: user_stability shouldn't really be here
@@ -731,7 +843,12 @@ class Implementation(object):
 		     'requires', 'metadata', 'download_sources', 'commands',
 		     'id', 'feed', 'version', 'released', 'bindings', 'machine']
 
+	quick_test_file = None
+	quick_test_mtime = None
+
 	def __init__(self, feed, id):
+		"""@type feed: L{ZeroInstallFeed}
+		@type id: str"""
 		assert id
 		self.feed = feed
 		self.id = id
@@ -748,16 +865,20 @@ class Implementation(object):
 		self.commands = {}
 
 	def get_stability(self):
+		"""@rtype: L{Stability}"""
 		return self.user_stability or self.upstream_stability or testing
 
 	def __str__(self):
+		"""@rtype: str"""
 		return self.id
 
 	def __repr__(self):
 		return "v%s (%s)" % (self.get_version(), self.id)
 
 	def __cmp__(self, other):
-		"""Newer versions come first"""
+		"""Newer versions come first
+		@type other: L{Implementation}
+		@rtype: int"""
 		d = cmp(other.version, self.version)
 		if d: return d
 		# If the version number is the same, just give a stable sort order, and
@@ -767,9 +888,12 @@ class Implementation(object):
 		return cmp(other.id, self.id)
 
 	def __hash__(self):
+		"""@rtype: int"""
 		return self.id.__hash__()
 
 	def __eq__(self, other):
+		"""@type other: L{Implementation}
+		@rtype: bool"""
 		return self is other
 
 	def __le__(self, other):
@@ -786,8 +910,8 @@ class Implementation(object):
 
 	def get_version(self):
 		"""Return the version as a string.
-		@see: L{format_version}
-		"""
+		@rtype: str
+		@see: L{format_version}"""
 		return format_version(self.version)
 
 	arch = property(lambda self: _join_arch(self.os, self.machine))
@@ -798,7 +922,8 @@ class Implementation(object):
 	requires_root_install = False
 
 	def _get_main(self):
-		""""@deprecated: use commands["run"] instead"""
+		""""@deprecated: use commands["run"] instead
+		@rtype: str"""
 		main = self.commands.get("run", None)
 		if main is not None:
 			return main.path
@@ -816,8 +941,7 @@ class Implementation(object):
 		"""Is this Implementation available locally?
 		(a local implementation, an installed distribution package, or a cached ZeroInstallImplementation)
 		@rtype: bool
-		@since: 0.53
-		"""
+		@since: 0.53"""
 		raise NotImplementedError("abstract")
 
 class DistributionImplementation(Implementation):
@@ -826,15 +950,22 @@ class DistributionImplementation(Implementation):
 	@ivar package_implementation: the <package-implementation> element that generated this impl (since 1.7)
 	@type package_implementation: L{qdom.Element}
 	@since: 0.28"""
-	__slots__ = ['distro', 'installed', 'package_implementation', 'distro_name']
+	__slots__ = ['distro', 'installed', 'package_implementation', 'distro_name', 'quick_test_file', 'quick_test_mtime']
 
 	def __init__(self, feed, id, distro, package_implementation = None, distro_name = None):
+		"""@type feed: L{ZeroInstallFeed}
+		@type id: str
+		@type distro: L{zeroinstall.injector.distro.Distribution}
+		@type package_implementation: L{zeroinstall.injector.qdom.Element} | None
+		@type distro_name: str | None"""
 		assert id.startswith('package:')
 		Implementation.__init__(self, feed, id)
 		self.distro = distro
 		self.installed = False
 		self.package_implementation = package_implementation
 		self.distro_name = distro_name or distro.name
+		self.quick_test_file = None
+		self.quick_test_mtime = None
 
 		if package_implementation:
 			for child in package_implementation.childNodes:
@@ -850,6 +981,8 @@ class DistributionImplementation(Implementation):
 		return not self.installed
 
 	def is_available(self, stores):
+		"""@type stores: L{zeroinstall.zerostore.Stores}
+		@rtype: bool"""
 		return self.installed
 
 class ZeroInstallImplementation(Implementation):
@@ -862,7 +995,10 @@ class ZeroInstallImplementation(Implementation):
 	distro_name = '0install'
 
 	def __init__(self, feed, id, local_path):
-		"""id can be a local path (string starting with /) or a manifest hash (eg "sha1=XXX")"""
+		"""id can be a local path (string starting with /) or a manifest hash (eg "sha1=XXX")
+		@type feed: L{ZeroInstallFeed}
+		@type id: str
+		@type local_path: str"""
 		assert not id.startswith('package:'), id
 		Implementation.__init__(self, feed, id)
 		self.size = None
@@ -874,15 +1010,24 @@ class ZeroInstallImplementation(Implementation):
 	dependencies = property(lambda self: dict([(x.interface, x) for x in self.requires
 						   if isinstance(x, InterfaceRestriction)]))
 
-	def add_download_source(self, url, size, extract, start_offset = 0, type = None):
-		"""Add a download source."""
-		self.download_sources.append(DownloadSource(self, url, size, extract, start_offset, type))
+	def add_download_source(self, url, size, extract, start_offset = 0, type = None, dest = None):
+		"""Add a download source.
+		@type url: str
+		@type size: int
+		@type extract: str
+		@type start_offset: int
+		@type type: str | None
+		@type dest: str | None"""
+		self.download_sources.append(DownloadSource(self, url, size, extract, start_offset, type, dest))
 
 	def set_arch(self, arch):
+		"""@type arch: str"""
 		self.os, self.machine = _split_arch(arch)
 	arch = property(lambda self: _join_arch(self.os, self.machine), set_arch)
 
 	def is_available(self, stores):
+		"""@type stores: L{zeroinstall.zerostore.Stores}
+		@rtype: bool"""
 		if self.local_path is not None:
 			return os.path.exists(self.local_path)
 		if self.digests:
@@ -910,6 +1055,7 @@ class Interface(object):
 	last_checked = property(lambda self: self._main_feed.last_checked)
 
 	def __init__(self, uri):
+		"""@type uri: str"""
 		assert uri
 		if uri.startswith('http:') or uri.startswith('https:') or os.path.isabs(uri):
 			self.uri = uri
@@ -930,6 +1076,7 @@ class Interface(object):
 		self.stability_policy = None
 
 	def get_name(self):
+		"""@rtype: str"""
 		from zeroinstall.injector.iface_cache import iface_cache
 		feed = iface_cache.get_feed(self.uri)
 		if feed:
@@ -937,9 +1084,11 @@ class Interface(object):
 		return '(' + os.path.basename(self.uri) + ')'
 
 	def __repr__(self):
+		"""@rtype: str"""
 		return _("<Interface %s>") % self.uri
 
 	def set_stability_policy(self, new):
+		"""@type new: L{Stability}"""
 		assert new is None or isinstance(new, Stability)
 		self.stability_policy = new
 
@@ -970,14 +1119,16 @@ def _merge_attrs(attrs, item):
 	"""Add each attribute of item to a copy of attrs and return the copy.
 	@type attrs: {str: str}
 	@type item: L{qdom.Element}
-	@rtype: {str: str}
-	"""
+	@rtype: {str: str}"""
 	new = attrs.copy()
 	for a in item.attrs:
 		new[str(a)] = item.attrs[a]
 	return new
 
 def _get_long(elem, attr_name):
+	"""@type elem: L{zeroinstall.injector.qdom.Element}
+	@type attr_name: str
+	@rtype: int"""
 	val = elem.getAttribute(attr_name)
 	if val is not None:
 		try:
@@ -1014,7 +1165,8 @@ class ZeroInstallFeed(object):
 		"""Create a feed object from a DOM.
 		@param feed_element: the root element of a feed file
 		@type feed_element: L{qdom.Element}
-		@param local_path: the pathname of this local feed, or None for remote feeds"""
+		@param local_path: the pathname of this local feed, or None for remote feeds
+		@type local_path: str | None"""
 		self.local_path = local_path
 		self.implementations = {}
 		self.name = None
@@ -1036,7 +1188,8 @@ class ZeroInstallFeed(object):
 		if feed_element is None:
 			return			# XXX subclass?
 
-		assert feed_element.name in ('interface', 'feed'), "Root element should be <interface>, not %s" % feed_element
+		if feed_element.name not in ('interface', 'feed'):
+			raise SafeException("Root element should be <interface>, not <%s>" % feed_element.name)
 		assert feed_element.uri == XMLNS_IFACE, "Wrong namespace on root element: %s" % feed_element.uri
 
 		main = feed_element.getAttribute('main')
@@ -1211,6 +1364,18 @@ class ZeroInstallFeed(object):
 			impl.bindings = bindings
 			impl.requires = depends
 
+			def extract_file_source(elem):
+				url = elem.getAttribute('href')
+				if not url:
+					raise InvalidInterface(_("Missing href attribute on <file>"))
+				dest = elem.getAttribute('dest')
+				if not dest:
+					raise InvalidInterface(_("Missing dest attribute on <file>"))
+				size = elem.getAttribute('size')
+				if not size:
+					raise InvalidInterface(_("Missing size attribute on <file>"))
+				return FileSource(url, dest, int(size))
+
 			for elem in item.childNodes:
 				if elem.uri != XMLNS_IFACE: continue
 				if elem.name == 'archive':
@@ -1223,7 +1388,11 @@ class ZeroInstallFeed(object):
 					impl.add_download_source(url = url, size = int(size),
 							extract = elem.getAttribute('extract'),
 							start_offset = _get_long(elem, 'start-offset'),
-							type = elem.getAttribute('type'))
+							type = elem.getAttribute('type'),
+							dest = elem.getAttribute('dest'))
+				elif elem.name == 'file':
+					impl.download_sources.append(extract_file_source(elem))
+
 				elif elem.name == 'manifest-digest':
 					for aname, avalue in elem.attrs.items():
 						if ' ' not in aname:
@@ -1241,7 +1410,10 @@ class ZeroInstallFeed(object):
 							recipe.steps.append(DownloadSource(None, url = url, size = int(size),
 									extract = recipe_step.getAttribute('extract'),
 									start_offset = _get_long(recipe_step, 'start-offset'),
-									type = recipe_step.getAttribute('type')))
+									type = recipe_step.getAttribute('type'),
+									dest = recipe_step.getAttribute('dest')))
+						elif recipe_step.name == 'file':
+							recipe.steps.append(extract_file_source(recipe_step))
 						elif recipe_step.uri == XMLNS_IFACE and recipe_step.name == 'rename':
 							source = recipe_step.getAttribute('source')
 							if not source:
@@ -1250,6 +1422,11 @@ class ZeroInstallFeed(object):
 							if not dest:
 								raise InvalidInterface(_("Missing dest attribute on <rename>"))
 							recipe.steps.append(RenameStep(source=source, dest=dest))
+						elif recipe_step.uri == XMLNS_IFACE and recipe_step.name == 'remove':
+							path = recipe_step.getAttribute('path')
+							if not path:
+								raise InvalidInterface(_("Missing path attribute on <remove>"))
+							recipe.steps.append(RemoveStep(path=path))
 						else:
 							logger.info(_("Unknown step '%s' in recipe; skipping recipe"), recipe_step.name)
 							break
@@ -1267,6 +1444,7 @@ class ZeroInstallFeed(object):
 		"""Does this feed contain any <pacakge-implementation> elements?
 		i.e. is it worth asking the package manager for more information?
 		@return: the URL of the virtual feed, or None
+		@rtype: str
 		@since: 0.49"""
 		if self._package_implementations:
 			return "distribution:" + self.url
@@ -1295,6 +1473,7 @@ class ZeroInstallFeed(object):
 		return best_impls
 
 	def get_name(self):
+		"""@rtype: str"""
 		return self.name or '(' + os.path.basename(self.url) + ')'
 
 	def __repr__(self):
@@ -1314,7 +1493,9 @@ class ZeroInstallFeed(object):
 		self.metadata.append(elem)
 
 	def get_metadata(self, uri, name):
-		"""Return a list of interface metadata elements with this name and namespace URI."""
+		"""Return a list of interface metadata elements with this name and namespace URI.
+		@type uri: str
+		@type name: str"""
 		return [m for m in self.metadata if m.name == name and m.uri == uri]
 
 	@property
@@ -1329,6 +1510,7 @@ class ZeroInstallFeed(object):
 		"""Return the URI of the interface that replaced the one with the URI of this feed's URL.
 		This is the value of the feed's <replaced-by interface'...'/> element.
 		@return: the new URI, or None if it hasn't been replaced
+		@rtype: str | None
 		@since: 1.7"""
 		for child in self.metadata:
 			if child.uri == XMLNS_IFACE and child.name == 'replaced-by':
@@ -1363,6 +1545,7 @@ if sys.version_info[0] > 2:
 	# it uses upper-case escapes and we use lower-case ones...
 	def unescape(uri):
 		"""Convert each %20 to a space, etc.
+		@type uri: str
 		@rtype: str"""
 		uri = uri.replace('#', '/')
 		if '%' not in uri: return uri
@@ -1372,6 +1555,7 @@ if sys.version_info[0] > 2:
 
 	def escape(uri):
 		"""Convert each space to %20, etc
+		@type uri: str
 		@rtype: str"""
 		return re.sub(b'[^-_.a-zA-Z0-9]',
 			lambda match: ('%%%02x' % ord(match.group(0))).encode('ascii'),
@@ -1381,6 +1565,7 @@ if sys.version_info[0] > 2:
 		"""Convert each space to %20, etc
 		: is preserved and / becomes #. This makes for nicer strings,
 		and may replace L{escape} everywhere in future.
+		@type uri: str
 		@rtype: str"""
 		if os.name == "posix":
 			# Only preserve : on Posix systems
@@ -1396,6 +1581,7 @@ else:
 
 	def unescape(uri):
 		"""Convert each %20 to a space, etc.
+		@type uri: str
 		@rtype: str"""
 		uri = uri.replace('#', '/')
 		if '%' not in uri: return uri
@@ -1405,6 +1591,7 @@ else:
 
 	def escape(uri):
 		"""Convert each space to %20, etc
+		@type uri: str
 		@rtype: str"""
 		return re.sub('[^-_.a-zA-Z0-9]',
 			lambda match: '%%%02x' % ord(match.group(0)),
@@ -1414,6 +1601,7 @@ else:
 		"""Convert each space to %20, etc
 		: is preserved and / becomes #. This makes for nicer strings,
 		and may replace L{escape} everywhere in future.
+		@type uri: str
 		@rtype: str"""
 		if os.name == "posix":
 			# Only preserve : on Posix systems
@@ -1431,7 +1619,8 @@ def escape_interface_uri(uri):
 	"file:///root/feed.xml" becomes ["file", "root__feed.xml"]
 	The number of components is determined by the scheme (three for http, two for file).
 	Uses L{support.escaping.underscore_escape} to escape each component.
-	"""
+	@type uri: str
+	@rtype: [str]"""
 	if uri.startswith('http://') or uri.startswith('https://'):
 		scheme, rest = uri.split('://', 1)
 		parts = rest.split('/', 1)
@@ -1447,9 +1636,9 @@ def canonical_iface_uri(uri):
 	A "file:///foo" URI is converted to "/foo".
 	An "alias:prog" URI expands to the URI in the 0alias script
 	Otherwise, return it unmodified.
+	@type uri: str
 	@rtype: str
-	@raise SafeException: if uri isn't valid
-	"""
+	@raise SafeException: if uri isn't valid"""
 	if uri.startswith('http://') or uri.startswith('https://'):
 		if uri.count("/") < 3:
 			raise SafeException(_("Missing / after hostname in URI '%s'") % uri)
