@@ -35,17 +35,126 @@ callbacks. See the Task class (below) for more information.
 # Copyright (C) 2009, Thomas Leonard
 # See the README file for details, or visit http://0install.net.
 
-from zeroinstall import _, support, gobject, logger
+from zeroinstall import _, support, logger
 import sys
 
+tulip = None
+_loop = None
+def get_loop():
+	global _loop, tulip
+	if _loop:
+		return _loop
+
+	gi_version = (0, 0, 0)
+	if sys.version_info[0] > 2:
+		try:
+			from gi import version_info as gi_version
+		except ImportError:
+			pass			 	# Not sure if this can happen
+		try:
+			from gi.repository import GObject as gobject
+			try:
+				from gi.repository import GLib as glib
+			except ImportError:
+				glib = gobject		# Old gobject includes glib
+		except ImportError:
+			gobject = None
+			glib = None
+	else:
+		import gobject
+		glib = gobject
+
+	if glib:
+		glib.threads_init()
+
+		class _Handler(object):
+			def cancel(self):
+				if self.tag is not None:
+					glib.source_remove(self.tag)
+					self.tag = None
+
+		class loop(object):
+			@staticmethod
+			def call_soon_threadsafe(cb):
+				def wrapper():
+					cb()
+					return False
+				glib.idle_add(wrapper)
+
+			call_soon = call_soon_threadsafe
+
+			@staticmethod
+			def call_repeatedly(interval, cb):
+				h = _Handler()
+				def wrapper():
+					assert h.tag is not None
+					cb()
+					return True
+				h.tag = glib.timeout_add(int(interval * 1000), wrapper)
+				return h
+
+			@staticmethod
+			def call_later(delay, cb):
+				"""@type delay: float"""
+				def wrapper():
+					cb()
+					return False
+				glib.timeout_add(int(delay * 1000), wrapper)
+
+			@staticmethod
+			def add_reader(fd, cb, *args):
+				"""@type fd: int
+				@rtype: L{_Handler}"""
+				h = _Handler()
+				def wrapper(src, cond):
+					cb(*args)
+					return True
+				if gi_version < (3, 7, 3):
+					h.tag = glib.io_add_watch(fd, glib.IO_IN | glib.IO_HUP, wrapper)
+				else:
+					h.tag = glib.io_add_watch(fd, glib.PRIORITY_DEFAULT, glib.IO_IN | glib.IO_HUP, wrapper)
+
+				return h
+
+			@staticmethod
+			def add_writer(fd, cb, *args):
+				h = _Handler()
+				def wrapper(src, cond):
+					cb(*args)
+					return True
+				if gi_version < (3, 7, 3):
+					h.tag = glib.io_add_watch(fd, glib.IO_OUT | glib.IO_HUP, wrapper)
+				else:
+					h.tag = glib.io_add_watch(fd, glib.PRIORITY_DEFAULT, glib.IO_OUT | glib.IO_HUP, wrapper)
+				return h
+
+		loop.glib = glib
+		loop.gobject = gobject
+	else:
+		try:
+			import tulip
+		except ImportError:
+			# Delay the error until we actually need a mainloop
+			class Fail(object):
+				def __getattr__(self, x):
+					raise Exception("No mainloop available: install python3-gi or tulip")
+			tulip = loop = Fail()
+		else:
+			loop = tulip.get_event_loop()
+			loop.glib = None
+
+	_loop = loop
+	return loop
+
 # The list of Blockers whose event has happened, in the order they were
 # triggered
 _run_queue = []
 
 def check(blockers, reporter = None):
 	"""See if any of the blockers have pending exceptions.
-	@param reporter: invoke this function on each error
-	If reporter is None, raise the first and log the rest."""
+	If reporter is None, raise the first and log the rest.
+	@type blockers: [L{Blocker}]
+	@param reporter: invoke this function on each error"""
 	ex = None
 	if isinstance(blockers, Blocker):
 		blockers = (blockers,)
@@ -65,7 +174,7 @@ def check(blockers, reporter = None):
 	if ex:
 		support.raise_with_traceback(ex[0], ex[1])
 
-class Blocker:
+class Blocker(object):
 	"""A Blocker object starts life with 'happened = False'. Tasks can
 	ask to be suspended until 'happened = True'. The value is changed
 	by a call to trigger().
@@ -99,6 +208,7 @@ class Blocker:
 	exception = None
 
 	def __init__(self, name):
+		"""@type name: str"""
 		self.happened = False		# False until event triggered
 		self._zero_lib_tasks = set()	# Tasks waiting on this blocker
 		self.name = name
@@ -129,19 +239,21 @@ class Blocker:
 	#def __del__(self):
 	#	if self.exception and not self.exception_read:
 	#		warn(_("Blocker %(blocker)s garbage collected without having it's exception read: %(exception)s"), {'blocker': self, 'exception': self.exception})
-	
+
 	def add_task(self, task):
 		"""Called by the schedular when a Task yields this
 		Blocker. If you override this method, be sure to still
-		call this method with Blocker.add_task(self)!"""
+		call this method with Blocker.add_task(self)!
+		@type task: L{Task}"""
 		assert task not in self._zero_lib_tasks, "Blocking on a single task twice: %s (%s)" % (task, self)
 		self._zero_lib_tasks.add(task)
-	
+
 	def remove_task(self, task):
 		"""Called by the schedular when a Task that was waiting for
-		this blocker is resumed."""
+		this blocker is resumed.
+		@type task: L{Task}"""
 		self._zero_lib_tasks.remove(task)
-	
+
 	def __repr__(self):
 		return "<Blocker:%s>" % self
 
@@ -153,44 +265,52 @@ class IdleBlocker(Blocker):
 	immediately triggers. An instance of this class is used internally
 	when a Task yields None."""
 	def add_task(self, task):
-		"""Also calls trigger."""
+		"""Also calls trigger.
+		@type task: L{Task}"""
 		Blocker.add_task(self, task)
 		self.trigger()
 
 class TimeoutBlocker(Blocker):
 	"""Triggers after a set number of seconds."""
 	def __init__(self, timeout, name):
-		"""Trigger after 'timeout' seconds (may be a fraction)."""
+		"""Trigger after 'timeout' seconds (may be a fraction).
+		@type timeout: float
+		@type name: str"""
 		Blocker.__init__(self, name)
-		gobject.timeout_add(int(timeout * 1000), self._timeout)
-	
+		get_loop().call_later(timeout, self._timeout)
+
 	def _timeout(self):
 		self.trigger()
 
-def _io_callback(src, cond, blocker):
+def _io_callback(blocker):
+	"""@type blocker: L{InputBlocker} | L{OutputBlocker}"""
+	blocker._tag.cancel()
 	blocker.trigger()
-	return False
 
 class InputBlocker(Blocker):
 	"""Triggers when os.read(stream) would not block."""
 	_tag = None
 	_stream = None
 	def __init__(self, stream, name):
+		"""@type stream: int
+		@type name: str"""
 		Blocker.__init__(self, name)
 		self._stream = stream
-	
+
 	def add_task(self, task):
+		"""@type task: L{Task}"""
 		Blocker.add_task(self, task)
 		if self._tag is None:
-			self._tag = gobject.io_add_watch(self._stream, gobject.IO_IN | gobject.IO_HUP,
-				_io_callback, self)
-	
+			self._tag = get_loop().add_reader(self._stream, _io_callback, self)
+
 	def remove_task(self, task):
+		"""@type task: L{Task}"""
 		Blocker.remove_task(self, task)
 		if not self._zero_lib_tasks:
-			gobject.source_remove(self._tag)
+			self._tag.cancel()
 			self._tag = None
 
+# Note: this isn't used within 0install
 class OutputBlocker(Blocker):
 	"""Triggers when os.write(stream) would not block."""
 	_tag = None
@@ -198,22 +318,21 @@ class OutputBlocker(Blocker):
 	def __init__(self, stream, name):
 		Blocker.__init__(self, name)
 		self._stream = stream
-	
+
 	def add_task(self, task):
 		Blocker.add_task(self, task)
 		if self._tag is None:
-			self._tag = gobject.io_add_watch(self._stream, gobject.IO_OUT | gobject.IO_HUP,
-				_io_callback, self)
-	
+			self._tag = get_loop().add_writer(self._stream, _io_callback, self)
+
 	def remove_task(self, task):
 		Blocker.remove_task(self, task)
 		if not self._zero_lib_tasks:
-			gobject.source_remove(self._tag)
+			self._tag.cancel()
 			self._tag = None
 
 _idle_blocker = IdleBlocker("(idle)")
 
-class Task:
+class Task(object):
 	"""Create a new Task when you have some long running function to
 	run in the background, but which needs to do work in 'chunks'.
 	Example:
@@ -237,14 +356,15 @@ class Task:
 	def __init__(self, iterator, name):
 		"""Call next(iterator) from a glib idle function. This function
 		can yield Blocker() objects to suspend processing while waiting
-		for events. name is used only for debugging."""
+		for events. name is used only for debugging.
+		@type name: str"""
 		self.iterator = iterator
 		self.finished = Blocker(name)
 		# Block new task on the idle handler...
 		_idle_blocker.add_task(self)
 		self._zero_blockers = (_idle_blocker,)
 		logger.info(_("Scheduling new task: %s"), self)
-	
+
 	def _resume(self):
 		# Remove from our blockers' queues
 		for blocker in self._zero_blockers:
@@ -286,17 +406,17 @@ class Task:
 		for blocker in new_blockers:
 			blocker.add_task(self)
 		self._zero_blockers = new_blockers
-	
+
 	def __repr__(self):
 		return "Task(%s)" % self.finished.name
-	
+
 	def __str__(self):
 		return self.finished.name
 
 # Must append to _run_queue right after calling this!
 def _schedule():
 	assert not _run_queue
-	gobject.idle_add(_handle_run_queue)
+	get_loop().call_soon(_handle_run_queue)
 
 def _handle_run_queue():
 	global _idle_blocker
@@ -325,13 +445,13 @@ def _handle_run_queue():
 	del _run_queue[0]
 
 	if _run_queue:
-		return True
-	return False
+		get_loop().call_soon(_handle_run_queue)
 
 def named_async(name):
 	"""Decorator that turns a generator function into a function that runs the
 	generator as a Task and returns the Task's finished blocker.
-	@param name: the name for the Task"""
+	@param name: the name for the Task
+	@type name: str"""
 	def deco(fn):
 		def run(*args, **kwargs):
 			return Task(fn(*args, **kwargs), name).finished
@@ -351,24 +471,32 @@ def wait_for_blocker(blocker):
 	"""Run a recursive mainloop until blocker is triggered.
 	@param blocker: event to wait on
 	@type blocker: L{Blocker}
-	@since: 0.53
-	"""
-	assert wait_for_blocker.loop is None	# Avoid recursion
+	@since: 0.53"""
+	assert wait_for_blocker.x is None	# Avoid recursion
+
+	loop = get_loop()
+	glib = loop.glib
 
 	if not blocker.happened:
 		def quitter():
 			yield blocker
-			wait_for_blocker.loop.quit()
+			if glib:
+				wait_for_blocker.x.quit()
+			else:
+				wait_for_blocker.x.set_result(None)
 		Task(quitter(), "quitter")
 
-		wait_for_blocker.loop = gobject.MainLoop()
+		wait_for_blocker.x = glib.MainLoop() if glib else tulip.Future()
 		try:
 			logger.debug(_("Entering mainloop, waiting for %s"), blocker)
-			wait_for_blocker.loop.run()
+			if glib:
+				wait_for_blocker.x.run()
+			else:
+				get_loop().run_until_complete(wait_for_blocker.x)
 		finally:
-			wait_for_blocker.loop = None
+			wait_for_blocker.x = None
 
 		assert blocker.happened, "Someone quit the main loop!"
 
 	check(blocker)
-wait_for_blocker.loop = None
+wait_for_blocker.x = None
